from __future__ import annotations

import gradio as gr

"""
Each entry is a tuple, where the first element is a label, and the second is either a single benchmark or a group of benchmarks.

Example:
[
    ("First Benchmark", dict(value="MTEB(something)", icon="icon_url")),
    ("Group of Benchmarks",
        [
            ("Second Benchmark", dict(value="MTEB(something)", icon="icon_url")),
            ("Third Benchmark", dict(value="MTEB(something)", icon="icon_url")),
        ],
    ),
]
"""
BENCHMARK_ENTRIES = [
    (
        "Multilingual",
        dict(
            value="MTEB(Multilingual, v1)",
            icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-gui-globe.svg",
        ),
    ),
    (
        "English",
        dict(
            value="MTEB(eng, v2)",
            icon="https://github.com/lipis/flag-icons/raw/refs/heads/main/flags/4x3/us.svg",
        ),
    ),
    (
        "Image Benchmarks",
        [
            (
                "Images, Multilingual",
                dict(
                    value="MIEB(Multilingual)",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-gui-pictures.svg",
                ),
            ),
            (
                "Images, English",
                dict(
                    value="MIEB(eng)",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-gui-picture.svg",
                ),
            ),
            (
                "Images, Lite",
                dict(
                    value="MIEB(lite)",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-map-landscape.svg",
                ),
            ),
        ],
    ),
    (
        "Domain-Specific Benchmarks",
        [
            (
                "Code",
                dict(
                    value="MTEB(Code, v1)",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-tech-electronics.svg",
                ),
            ),
            (
                "Legal",
                dict(
                    value="MTEB(Law, v1)",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-map-library.svg",
                ),
            ),
            (
                "Medical",
                dict(
                    value="MTEB(Medical, v1)",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-map-hospital.svg",
                ),
            ),
            (
                "Chemical",
                dict(
                    value="ChemTEB",
                    icon="https://github.com/DennisSuitters/LibreICONS/raw/2d2172d15e3c6ca03c018629d60050e4b99e5c55/svg-color/libre-gui-purge.svg",
                ),
            ),
        ],
    ),
    (
        "Regional Benchmarks",
        [
            (
                "European",
                dict(
                    value="MTEB(Europe, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/eu.svg",
                ),
            ),
            (
                "Indic",
                dict(
                    value="MTEB(Indic, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/in.svg",
                ),
            ),
            (
                "Scandinavian",
                dict(
                    value="MTEB(Scandinavian, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/dk.svg",
                ),
            ),
        ],
    ),
    (
        "Language-specific Benchmarks",
        [
            (
                "Chinese",
                dict(
                    value="MTEB(cmn, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/cn.svg",
                ),
            ),
            (
                "German",
                dict(
                    value="MTEB(deu, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/de.svg",
                ),
            ),
            (
                "French",
                dict(
                    value="MTEB(fra, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/fr.svg",
                ),
            ),
            (
                "Japanese",
                dict(
                    value="MTEB(jpn, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/jp.svg",
                ),
            ),
            (
                "Korean",
                dict(
                    value="MTEB(kor, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/kr.svg",
                ),
            ),
            (
                "Polish",
                dict(
                    value="MTEB(pol, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/pl.svg",
                ),
            ),
            (
                "Russian",
                dict(
                    value="MTEB(rus, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/ru.svg",
                ),
            ),
            (
                "Farsi (BETA)",
                dict(
                    value="MTEB(fas, beta)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/ir.svg",
                ),
            ),
        ],
    ),
    (
        "Miscellaneous",
        [
            ("BEIR", dict(value="BEIR", icon=None)),
            ("BEIR-NL", dict(value="BEIR-NL", icon=None)),
            ("BRIGHT", dict(value="BRIGHT", icon=None)),
            ("BRIGHT (long)", dict(value="BRIGHT (long)", icon=None)),
            ("BuiltBench (eng)", dict(value="BuiltBench(eng)", icon=None)),
            ("Code Information Retrieval", dict(value="CoIR", icon=None)),
            ("Instruction Following", dict(value="FollowIR", icon=None)),
            ("Long-context Retrieval", dict(value="LongEmbed", icon=None)),
            ("MINERSBitextMining", dict(value="MINERSBitextMining", icon=None)),
            ("NanoBEIR", dict(value="NanoBEIR", icon=None)),
            ("Reasoning retrieval", dict(value="RAR-b", icon=None)),
        ],
    ),
    (
        "Legacy",
        [
            (
                "English Legacy",
                dict(
                    value="MTEB(eng, v1)",
                    icon="https://github.com/lipis/flag-icons/raw/260c91531be024944c6514130c5defb2ebb02b7d/flags/4x3/gb.svg",
                ),
            ),
        ],
    ),
]


def _create_button(i, label, entry, state, label_to_value, **kwargs):
    val = entry["value"]
    label_to_value[label] = val
    button = gr.Button(
        label,
        variant="secondary" if i != 0 else "primary",
        icon=entry["icon"],
        key=f"{i}_button_{val}",
        elem_classes="text-white",
        **kwargs,
    )

    def _update_variant(state, label) -> gr.Button:
        if state == label_to_value[label]:
            return gr.Button(variant="primary")
        else:
            return gr.Button(variant="secondary")

    def _update_value(label) -> str:
        return label_to_value[label]

    state.change(_update_variant, inputs=[state, button], outputs=[button])
    button.click(_update_value, outputs=[state], inputs=[button])
    return button


def make_selector(entries: list[tuple[str, dict | list]]) -> tuple[gr.State, gr.Column]:
    if not entries:
        raise ValueError("No entries were specified, can't build selector.")
    label_to_value = {}
    state = None
    with gr.Column() as column:
        for i, (label, entry) in enumerate(entries):
            if i == 0:
                if isinstance(entry, dict):
                    state = gr.State(entry["value"])
                else:
                    _label, _entry = entry[0]
                    state = gr.State(_entry["value"])
            if isinstance(entry, dict):
                button = _create_button(
                    i, label, entry, state, label_to_value, size="lg"
                )
            else:
                gr.Markdown(f"### **{label}**")
                for sub_label, sub_entry in entry:
                    button = _create_button(  # noqa: F841
                        i, sub_label, sub_entry, state, label_to_value, size="md"
                    )

    return state, column



"""Tests for the ModelResult class"""

from __future__ import annotations

import pandas as pd
import pytest

import mteb
from mteb.load_results import ModelResult
from mteb.load_results.task_results import TaskResult

# TODO: v2 ^ we probably want to refactor such that this import looks like
# from mteb.results import ModelResult, TaskResults


@pytest.fixture
def model_result() -> ModelResult:
    task_result = TaskResult(
        dataset_revision="1.0",
        task_name="BornholmBitextMining",  # just dummy results
        mteb_version="1.0.0",
        evaluation_time=100,
        scores={
            "train": [
                {
                    "main_score": 0.5,
                    "hf_subset": "en-de",
                    "languages": ["eng-Latn", "deu-Latn"],
                },
                {
                    "main_score": 0.6,
                    "hf_subset": "en-fr",
                    "languages": ["eng-Latn", "fra-Latn"],
                },
            ]
        },
    )

    return ModelResult(
        model_name="mock_model",
        model_revision="dummy",
        task_results=[task_result],
    )


def test_indexing(model_result: ModelResult) -> None:
    res = model_result[0]
    assert isinstance(res, TaskResult), (
        "indexing into the list should return a ModelResult"
    )


def test_utility_properties(
    model_result: ModelResult,
) -> None:
    mr = model_result
    assert isinstance(mr.task_names, list) and isinstance(mr.task_names[0], str)
    assert (
        isinstance(mr.languages, list)
        and isinstance(mr.languages[0], str)
        and "eng" in mr.languages  # known from mock data
    )
    assert isinstance(mr.task_types, list) and isinstance(mr.task_types[0], str)
    assert isinstance(mr.domains, list) and isinstance(mr.domains[0], str)


def test_select_tasks(
    model_result: ModelResult,
) -> None:
    tasks = [mteb.get_task("BornholmBitextMining")]
    mr = model_result.select_tasks(tasks=tasks)
    task_names = mr.task_names
    assert isinstance(task_names, list)
    assert len(task_names) == 1
    assert task_names[0] == "BornholmBitextMining"


def test_to_dataframe(
    model_result: ModelResult,  # noqa: F811
) -> None:
    mr = model_result
    required_columns = [
        "model_name",
        "task_name",
        "task_name",
        "score",
        "subset",
        "split",
    ]
    t1 = mr.to_dataframe(aggregation_level="subset", format="long")
    assert isinstance(t1, pd.DataFrame)
    assert all(col in t1.columns for col in required_columns), "Columns are missing"
    assert t1.shape[0] > 0, "Results table is empty"

    t2 = mr.to_dataframe(aggregation_level="split", format="long")
    assert all(
        col in t2.columns for col in required_columns if col not in ["subset"]
    ), "Columns are missing"
    assert "subset" not in t2.columns, "Subset column should not be present"
    assert t1.shape[0] >= t2.shape[0], (
        "Aggregation level 'split' should have more rows than 'subset'"
    )

    t3 = mr.to_dataframe(aggregation_level="task", format="long")
    assert all(
        col in t3.columns for col in required_columns if col not in ["subset", "split"]
    ), "Columns are missing"
    assert "subset" not in t3.columns, "Subset column should not be present"
    assert "split" not in t3.columns, "Split column should not be present"
    assert t2.shape[0] >= t3.shape[0], (
        "Aggregation level 'task' should have more rows than 'split'"
    )

    t4_wide = mr.to_dataframe(aggregation_level="task", format="wide")
    t4_long = mr.to_dataframe(aggregation_level="task", format="long")
    assert isinstance(t4_wide, pd.DataFrame)

    # we know it is only one task
    assert t4_wide[mr.model_name].tolist()[0] == t4_long["score"][0], (
        "The scores in wide and long format should be the same"
    )



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskImageClassification import AbsTaskImageClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class StanfordCarsClassification(AbsTaskImageClassification):
    metadata = TaskMetadata(
        name="StanfordCars",
        description="Classifying car images from 196 makes.",
        reference="https://pure.mpg.de/rest/items/item_2029263/component/file_2029262/content",
        dataset={
            "path": "isaacchung/StanfordCars",
            "revision": "09ffe9bc7864d3f1e851529e5c4b7e05601a04fb",
        },
        type="ImageClassification",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2013-01-01",
            "2013-04-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation="""@inproceedings{Krause2013CollectingAL,
        title={Collecting a Large-scale Dataset of Fine-grained Cars},
        author={Jonathan Krause and Jia Deng and Michael Stark and Li Fei-Fei},
        year={2013},
        url={https://api.semanticscholar.org/CorpusID:16632981}
        }""",
        descriptive_stats={
            "n_samples": {"test": 8041},
            "avg_character_length": {"test": 431.4},
        },
    )



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyMultiChoice import AbsTaskAny2AnyMultiChoice
from mteb.abstasks.TaskMetadata import TaskMetadata


class BLINKIT2IMultiChoice(AbsTaskAny2AnyMultiChoice):
    metadata = TaskMetadata(
        name="BLINKIT2IMultiChoice",
        description="Retrieve images based on images and specific retrieval instructions.",
        reference="https://arxiv.org/abs/2404.12390",
        dataset={
            "path": "JamieSJS/blink-it2i-multi",
            "revision": "a9f994925551c14503d00d86f1307bac6e2ead6a",
            "trust_remote_code": True,
        },
        type="VisionCentricQA",
        category="it2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2018-01-01", "2018-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation="""@article{fu2024blink,
        title={Blink: Multimodal large language models can see but not perceive},
        author={Fu, Xingyu and Hu, Yushi and Li, Bangzheng and Feng, Yu and Wang, Haoyu and Lin, Xudong and Roth, Dan and Smith, Noah A and Ma, Wei-Chiu and Krishna, Ranjay},
        journal={arXiv preprint arXiv:2404.12390},
        year={2024}
        }""",
        descriptive_stats={
            "n_samples": {"test": 1206},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 0.0,
                    "num_documents": 804,
                    "num_queries": 402,
                    "average_relevant_docs_per_query": 1,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskClustering import AbsTaskClustering
from mteb.abstasks.TaskMetadata import TaskMetadata


class BuiltBenchClusteringS2S(AbsTaskClustering):
    metadata = TaskMetadata(
        name="BuiltBenchClusteringS2S",
        description="Clustering of built asset names/titles based on categories identified within industry classification systems such as IFC, Uniclass, etc.",
        reference="https://arxiv.org/abs/2411.12056",
        dataset={
            "path": "mehrzad-shahin/BuiltBench-clustering-s2s",
            "revision": "1aaeb2ece89ea0a8c64e215c95c4cfaf7e891149",
        },
        type="Clustering",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2024-06-01", "2024-11-30"),
        domains=["Engineering", "Written"],
        task_subtypes=["Thematic clustering"],
        license="cc-by-nd-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="""@article{shahinmoghadam2024benchmarking,
    title={Benchmarking pre-trained text embedding models in aligning built asset information},
    author={Shahinmoghadam, Mehrzad and Motamedi, Ali},
    journal={arXiv preprint arXiv:2411.12056},
    year={2024}
}""",
        prompt="Identify the category of the built asset entities based on the names or titles",
    )



from __future__ import annotations

from .CQADupStackNLRetrieval import CQADupstackNLRetrieval
from .CQADupStackRetrieval import CQADupstackRetrieval
from .CQADupStackRetrievalFa import CQADupstackRetrievalFa
from .STS17MultilingualVisualSTS import (
    STS17MultilingualVisualSTSEng,
    STS17MultilingualVisualSTSMultilingual,
)
from .STSBenchmarkMultilingualVisualSTS import (
    STSBenchmarkMultilingualVisualSTSEng,
    STSBenchmarkMultilingualVisualSTSMultilingual,
)
from .SynPerChatbotConvSAClassification import SynPerChatbotConvSAClassification

__all__ = [
    "CQADupstackRetrieval",
    "CQADupstackRetrievalFa",
    "SynPerChatbotConvSAClassification",
    "CQADupstackNLRetrieval",
    "STS17MultilingualVisualSTSEng",
    "STS17MultilingualVisualSTSMultilingual",
    "STSBenchmarkMultilingualVisualSTSEng",
    "STSBenchmarkMultilingualVisualSTSMultilingual",
]



from __future__ import annotations

import json
import logging
import os
import warnings
from collections import defaultdict
from time import time
from typing import Any

import tqdm
from datasets import Dataset, Features, Value, load_dataset

from mteb.encoder_interface import Encoder

from ..evaluation.evaluators import utils
from ..evaluation.evaluators.InstructionRetrievalEvaluator import (
    InstructionRetrievalEvaluator,
)
from .AbsTask import AbsTask
from .AbsTaskRetrieval import HFDataLoader
from .TaskMetadata import DescriptiveStatistics

logger = logging.getLogger(__name__)


# Adapted from https://github.com/beir-cellar/beir/blob/f062f038c4bfd19a8ca942a9910b1e0d218759d4/beir/datasets/data_loader_hf.py#L10
class HFDataLoaderInstructions(HFDataLoader):
    def __init__(
        self,
        hf_repo: str | None = None,
        hf_repo_qrels: str | None = None,
        data_folder: str | None = None,
        prefix: str | None = None,
        corpus_file: str = "corpus.jsonl",
        query_file: str = "queries.jsonl",
        qrels_folder: str = "qrels",
        qrels_file: str = "",
        streaming: bool = False,
        keep_in_memory: bool = False,
        trust_remote_code: bool = False,
    ):
        self.corpus = {}
        self.queries = {}
        self.qrels = {}
        self.og_instructions = {}
        self.changed_instructions = {}
        self.top_ranked = {}
        self.hf_repo = hf_repo
        if hf_repo:
            # By default fetch qrels from same repo not a second repo with "-qrels" like in original
            self.hf_repo_qrels = hf_repo_qrels if hf_repo_qrels else hf_repo
        else:
            # data folder would contain these files:
            # (1) fiqa/corpus.jsonl  (format: jsonlines)
            # (2) fiqa/queries.jsonl (format: jsonlines)
            # (3) fiqa/qrels/test.tsv (format: tsv ("\t"))
            if prefix:
                query_file = prefix + "-" + query_file
                qrels_folder = prefix + "-" + qrels_folder

            self.corpus_file = (
                os.path.join(data_folder, corpus_file) if data_folder else corpus_file
            )
            self.query_file = (
                os.path.join(data_folder, query_file) if data_folder else query_file
            )
            self.qrels_folder = (
                os.path.join(data_folder, qrels_folder) if data_folder else None
            )
            self.qrels_file = qrels_file
        self.streaming = streaming
        self.keep_in_memory = keep_in_memory
        self.trust_remote_code = trust_remote_code

    def load(
        self, split="test"
    ) -> tuple[
        Dataset,
        Dataset,
        dict[str, dict[str, int]],
        dict[str, dict[str, int]],
        Dataset,
    ]:
        if not self.hf_repo:
            self.og_qrels_file = os.path.join(self.qrels_folder + "_og", split + ".tsv")
            self.changed_qrels_file = os.path.join(
                self.qrels_folder + "_changed", split + ".tsv"
            )
            self.check(fIn=self.corpus_file, ext="jsonl")
            self.check(fIn=self.query_file, ext="jsonl")
            self.check(fIn=self.og_qrels_file, ext="tsv")
            self.check(fIn=self.changed_qrels_file, ext="tsv")

        if not len(self.corpus):
            logger.info("Loading Corpus...")
            self._load_corpus()
            logger.info("Loaded %d %s Documents.", len(self.corpus), split.upper())
            logger.info("Doc Example: %s", self.corpus[0])

        if not len(self.queries):
            logger.info("Loading Queries...")
            self._load_queries()

        self._load_qrels(split, changed=False)
        self._load_qrels(split, changed=True)
        # filter queries with no qrels
        og_qrels_dict = defaultdict(dict)
        changed_qrels_dict = defaultdict(dict)

        def qrels_dict_init(row):
            og_qrels_dict[row["query-id"]][row["corpus-id"]] = int(row["score"])

        def qrels_changed_dict_init(row):
            changed_qrels_dict[row["query-id"]][row["corpus-id"]] = int(row["score"])

        self.changed_qrels.map(qrels_dict_init)
        self.og_qrels.map(qrels_changed_dict_init)
        self.og_qrels = og_qrels_dict
        self.changed_qrels = changed_qrels_dict
        self.queries = self.queries.filter(lambda x: x["id"] in self.og_qrels)
        logger.info("Loaded %d %s Queries.", len(self.queries), split.upper())
        logger.info("Query Example: %s", self.queries[0])

        # load top_ranked
        self.load_top_ranked()

        return (
            self.corpus,
            self.queries,
            self.og_qrels,
            self.changed_qrels,
            self.top_ranked,
        )

    def load_top_ranked(self) -> None:
        if self.hf_repo:
            top_ranked_ds = load_dataset(
                self.hf_repo,
                "top_ranked",
                keep_in_memory=self.keep_in_memory,
                streaming=self.streaming,
            )
        else:
            top_ranked_ds = load_dataset(
                "json",
                data_files=self.top_ranked_file,
                streaming=self.streaming,
                keep_in_memory=self.keep_in_memory,
            )
        top_ranked_ds = next(iter(top_ranked_ds.values()))  # get first split
        top_ranked_ds = top_ranked_ds.cast_column("qid", Value("string"))
        top_ranked_ds = top_ranked_ds.cast_column("pid", Value("string"))
        top_ranked_ds = top_ranked_ds.remove_columns(
            [col for col in top_ranked_ds.column_names if col not in ["qid", "pid"]]
        )
        self.top_ranked = top_ranked_ds

    def _load_queries(self):
        if self.hf_repo:
            queries_ds = load_dataset(
                self.hf_repo,
                "queries",
                keep_in_memory=self.keep_in_memory,
                streaming=self.streaming,
            )
        else:
            queries_ds = load_dataset(
                "json",
                data_files=self.query_file,
                streaming=self.streaming,
                keep_in_memory=self.keep_in_memory,
            )
        queries_ds = next(iter(queries_ds.values()))  # get first split
        queries_ds = queries_ds.cast_column("_id", Value("string"))
        queries_ds = queries_ds.rename_column("_id", "id")
        queries_ds = queries_ds.remove_columns(
            [
                col
                for col in queries_ds.column_names
                if col
                not in [
                    "id",
                    "text",
                    "instruction_og",
                    "instruction_changed",
                    "keywords",
                    "short_query",
                ]
            ]
        )
        self.queries = queries_ds

    def _load_qrels(self, split, changed=False):
        if self.hf_repo:
            qrels_ds = load_dataset(
                self.hf_repo_qrels,
                "qrels_og" if not changed else "qrels_changed",
                keep_in_memory=self.keep_in_memory,
                streaming=self.streaming,
            )[split]
        else:
            qrels_file = self.og_qrels_file if not changed else self.changed_qrels_file
            qrels_ds = load_dataset(
                "csv",
                data_files=qrels_file,
                delimiter="\t",
                keep_in_memory=self.keep_in_memory,
            )
        features = Features(
            {
                "query-id": Value("string"),
                "corpus-id": Value("string"),
                "score": Value("float"),
            }
        )
        qrels_ds = qrels_ds.cast(features)

        if changed:
            self.changed_qrels = qrels_ds
        else:
            self.og_qrels = qrels_ds


class InstructionRetrievalDescriptiveStatistics(DescriptiveStatistics):
    """Descriptive statistics for Instruction Retrieval tasks

    Attributes:
        num_samples: Number of samples
        num_queries: Number of queries
        num_docs: Number of documents
        number_of_characters: Total number of symbols in the dataset

        min_document_length: Minimum length of documents
        average_document_length: Average length of documents
        max_document_length: Maximum length of documents
        unique_docs: Number of unique documents

        min_query_length: Minimum length of queries
        average_query_length: Average length of queries
        max_query_length: Maximum length of queries
        unique_queries: Number of unique queries

        min_instruction_length: Minimum length of instructions
        average_instruction_length: Average length of instructions
        max_instruction_length: Maximum length of instructions
        unique_instructions: Number of unique instructions

        min_changed_instruction_length: Minimum length of changed instructions
        average_changed_instruction_length: Average length of changed instructions
        max_changed_instruction_length: Maximum length of changed instructions
        unique_changed_instructions: Number of unique changed instructions

        min_average_relevant_docs_per_query: Minimum number of relevant docs per query
        average_relevant_docs_per_query: Average number of relevant docs per query
        max_average_relevant_docs_per_query: Maximum number of relevant docs per query

        min_average_top_ranked_per_query: Minimum number of top ranked docs per query
        average_top_ranked_per_query: Average number of top ranked docs per query
        max_average_top_ranked_per_query: Maximum number of top ranked docs per query
    """

    num_samples: int
    num_queries: int
    num_docs: int
    number_of_characters: int

    min_document_length: int
    average_document_length: float
    max_document_length: int
    unique_docs: int

    min_query_length: int
    average_query_length: float
    max_query_length: int
    unique_queries: int

    min_instruction_length: int
    average_instruction_length: float
    max_instruction_length: int
    unique_instructions: int

    min_changed_instruction_length: int
    average_changed_instruction_length: float
    max_changed_instruction_length: int
    unique_changed_instructions: int

    min_average_relevant_docs_per_query: float
    average_relevant_docs_per_query: float
    max_average_relevant_docs_per_query: float

    min_average_top_ranked_per_query: float
    average_top_ranked_per_query: float
    max_average_top_ranked_per_query: float


class AbsTaskInstructionRetrieval(AbsTask):
    """Abstract class for retrieval tasks that use instructions. An example from Core17 would be
        query: What is the ongoing status of The Three Gorges Project?
        instruction: A relevant document will provide the projected or actual date of completion of the project, its estimated or actual total cost, or the estimated or ongoing electrical output of the finished project. Discussions of the social, political, or ecological impact of the project are not relevant.

    Child-classes must implement the following properties:
    self.corpus = dict[corpus_id, dict[str, str]] #id => dict with document datas like title and text
    self.queries = dict[query_id, str] #id => query
    self.relevant_docs = dict[query_id, dict[corpus_id, int]]
    self.og_instructions = dict[str, str] query => original instruction
    self.changed_instructions = dict[str, str] query => changed instruction
    self.top_ranked = dict[query_id, list[corpus_id]] #id => list of top ranked document ids

    See https://arxiv.org/abs/2403.15246 for more details
    """

    abstask_prompt = "Retrieve text based on user query."

    def __init__(
        self,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.do_length_ablation = kwargs.get("do_length_ablation", False)
        if self.do_length_ablation:
            logger.info("Running length ablation also...")
        warnings.warn(
            "`AbsTaskInstructionRetrieval` will be merged with Retrieval in v2.0.0.",
            DeprecationWarning,
        )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        self.corpus, self.queries, self.og_relevant_docs, self.changed_relevant_docs = (
            {},
            {},
            {},
            {},
        )
        self.og_instructions, self.changed_instructions = {}, {}
        self.top_ranked = {}
        if self.do_length_ablation:
            self.keywords, self.short_instructions = {}, {}

        dataset_path = self.metadata_dict["dataset"]["path"]
        hf_repo_qrels = (
            dataset_path + "-qrels" if "clarin-knext" in dataset_path else None
        )
        for split in kwargs.get("eval_splits", self.metadata_dict["eval_splits"]):
            (
                corpus,
                queries,
                og_relevant_docs,
                changed_relevant_docs,
                top_ranked_init,
            ) = HFDataLoaderInstructions(
                hf_repo=dataset_path,
                hf_repo_qrels=hf_repo_qrels,
                streaming=False,
                keep_in_memory=False,
            ).load(split=split)

            # Conversion from DataSet
            top_ranked = defaultdict(list)
            [
                top_ranked[cur_inst["qid"]].append(cur_inst["pid"])
                for cur_inst in top_ranked_init
            ]
            og_instructions = {
                query["text"]: query["instruction_og"] for query in queries
            }
            changed_instructions = {
                query["text"]: query["instruction_changed"] for query in queries
            }
            if self.do_length_ablation:
                keywords = {query["text"]: query["keywords"] for query in queries}
                short_instructions = {
                    query["text"]: query["short_query"] for query in queries
                }
            queries = {query["id"]: query["text"] for query in queries}
            corpus = {
                doc["id"]: {"title": doc["title"], "text": doc["text"]}
                for doc in corpus
            }
            assert len(top_ranked) == len(queries), (
                f"Top ranked not loaded properly! Expected {len(self.queries)} but got {len(self.top_ranked)}."
            )

            (
                self.corpus[split],
                self.queries[split],
                self.og_relevant_docs[split],
                self.changed_relevant_docs[split],
            ) = corpus, queries, og_relevant_docs, changed_relevant_docs
            self.changed_instructions[split], self.og_instructions[split] = (
                changed_instructions,
                og_instructions,
            )
            self.top_ranked[split] = top_ranked

            if self.do_length_ablation:
                self.keywords[split], self.short_instructions[split] = (
                    keywords,
                    short_instructions,
                )

        self.data_loaded = True

    def _evaluate_subset_lang(
        self,
        retriever: InstructionRetrievalEvaluator,
        corpus: dict,
        queries: dict,
        og_relevant_docs: dict,
        changed_relevant_docs: dict,
        og_instructions: dict,
        changed_instructions: dict,
        top_ranked: dict,
        lang: str,
        split: str,
        keywords: dict | None = None,
        short_instructions: dict | None = None,
        **kwargs,
    ) -> dict[str, dict[str, float] | float]:
        corpus, queries = corpus[split], queries[split]
        og_relevant_docs, changed_relevant_docs = (
            og_relevant_docs[split],
            changed_relevant_docs[split],
        )
        og_instructions, changed_instructions = (
            og_instructions[split],
            changed_instructions[split],
        )

        top_ranked = top_ranked[split]
        kwargs["prediction_name"] = "og"  # for naming predictions, as needed
        scores_og, results_og = self._evaluate_subset(
            retriever,
            corpus,
            queries,
            og_relevant_docs,
            og_instructions,
            top_ranked,
            lang,
            **kwargs,
        )
        kwargs["prediction_name"] = "changed"  # for naming predictions, as needed
        scores_changed, results_changed = self._evaluate_subset(
            retriever,
            corpus,
            queries,
            changed_relevant_docs,
            changed_instructions,
            top_ranked,
            lang,
            **kwargs,
        )

        newly_irrelevant_qrels = self.create_qrel_diff(
            og_relevant_docs,
            changed_relevant_docs,
        )
        overall_changed_scores = utils.evaluate_change(
            results_og, results_changed, newly_irrelevant_qrels
        )

        overall_changed_scores["individual"] = {
            "original": scores_og,
            "changed": scores_changed,
        }

        if self.do_length_ablation:
            keywords, short_instructions = (
                keywords[split],
                short_instructions[split],
            )
            kwargs["prediction_name"] = "base"  # for naming predictions, as needed
            scores_base, results_base = self._evaluate_subset(
                retriever,
                corpus,
                queries,
                og_relevant_docs,
                defaultdict(str),
                top_ranked,
                lang,
                **kwargs,
            )
            kwargs["prediction_name"] = "keywords"  # for naming predictions, as needed
            scores_w_keywords_scores, scores_w_keywords_results = self._evaluate_subset(
                retriever,
                corpus,
                queries,
                og_relevant_docs,
                keywords,
                top_ranked,
                lang,
                **kwargs,
            )
            kwargs["prediction_name"] = (
                "short_instr"  # for naming predictions, as needed
            )
            (
                scores_w_short_instr_scores,
                scores_w_short_instr_result,
            ) = self._evaluate_subset(
                retriever,
                corpus,
                queries,
                og_relevant_docs,
                short_instructions,
                top_ranked,
                lang,
                **kwargs,
            )
            overall_changed_scores["length_ablation"] = {
                "keywords": scores_w_keywords_scores,
                "short_instructions": scores_w_short_instr_scores,
                "base": scores_base,
            }

        return overall_changed_scores

    def evaluate(
        self,
        model: Encoder,
        split: str = "test",
        subsets_to_run: list[str] | None = None,
        *,
        encode_kwargs: dict[str, Any] = {},
        **kwargs,
    ) -> dict[str, dict[str, Any]]:
        retriever = InstructionRetrievalEvaluator(
            retriever=model,
            task_name=self.metadata.name,
            encode_kwargs=encode_kwargs,
            **kwargs,
        )
        scores = {}
        if self.is_multilingual:
            hf_subsets = self.hf_subsets
            if subsets_to_run is not None:
                hf_subsets = [s for s in hf_subsets if s in subsets_to_run]

            for lang in hf_subsets:
                logger.info(f"Language: {lang}")
                scores[lang] = self._evaluate_subset_lang(
                    retriever,
                    corpus=self.corpus[lang],
                    queries=self.queries[lang],
                    og_relevant_docs=self.og_relevant_docs[lang],
                    changed_relevant_docs=self.changed_relevant_docs[lang],
                    og_instructions=self.og_instructions[lang],
                    changed_instructions=self.changed_instructions[lang],
                    top_ranked=self.top_ranked[lang],
                    lang=lang,
                    split=split,
                    keywords=self.keywords[lang] if self.do_length_ablation else None,
                    short_instructions=self.short_instructions[lang]
                    if self.do_length_ablation
                    else None,
                    **kwargs,
                )
                self._add_main_score(scores[lang])
        else:
            lang = "default"
            scores[lang] = self._evaluate_subset_lang(
                retriever,
                corpus=self.corpus,
                queries=self.queries,
                og_relevant_docs=self.og_relevant_docs,
                changed_relevant_docs=self.changed_relevant_docs,
                og_instructions=self.og_instructions,
                changed_instructions=self.changed_instructions,
                top_ranked=self.top_ranked,
                lang=lang,
                split=split,
                keywords=self.keywords if self.do_length_ablation else None,
                short_instructions=self.short_instructions
                if self.do_length_ablation
                else None,
                **kwargs,
            )
            self._add_main_score(scores[lang])

        return scores

    def _add_main_score(self, scores: dict[str, dict[str, float]]) -> None:
        scores["main_score"] = scores[self.metadata.main_score]

    def _evaluate_subset(
        self,
        retriever: InstructionRetrievalEvaluator,
        corpus: dict[str, dict[str, str]],
        queries: dict[str, str],
        relevant_docs: dict[str, dict[str, int]],
        instructions: dict[str, str],
        top_ranked: dict[str, list[str]],
        lang=None,
        **kwargs,
    ) -> tuple[dict[str, float], dict[str, dict[str, float]]]:
        start_time = time()

        # do the results by query and relevant docs only
        all_results = []
        for query_id in tqdm.tqdm(list(queries.keys()), leave=False, desc="Retrieving"):
            cur_queries = {query_id: queries[query_id]}
            cur_instructions = {queries[query_id]: instructions[queries[query_id]]}
            cur_docs = {
                key: value
                for (key, value) in corpus.items()
                if key in top_ranked[query_id]
            }
            all_results.append(
                retriever(
                    cur_docs, cur_queries, instructions=cur_instructions, qid=query_id
                )
            )

        # combine all the results (which are {'qid' -> {'doc_id' -> score} mappings)
        # we know all are unique qids, so we can smash together
        results = {k: v for d in all_results for k, v in d.items()}

        end_time = time()
        logger.info(f"Time taken to retrieve: {end_time - start_time:.2f} seconds")

        if kwargs.get("save_predictions", False):
            output_folder = kwargs.get("output_folder", "results")
            if not os.path.isdir(output_folder):
                os.makedirs(output_folder)
            top_k = kwargs.get("top_k", None)
            if top_k is not None:
                for qid in list(results.keys()):
                    doc_ids = set(
                        sorted(
                            results[qid], key=lambda x: results[qid][x], reverse=True
                        )[:top_k]
                    )
                    results[qid] = {
                        k: v for k, v in results[qid].items() if k in doc_ids
                    }
            if lang is None:
                qrels_save_path = (
                    f"{output_folder}/{self.metadata_dict['name']}_predictions.json"
                )
            else:
                qrels_save_path = f"{output_folder}/{self.metadata_dict['name']}_{lang}_predictions.json"

            if kwargs.get("prediction_name", None):
                qrels_save_path = qrels_save_path.replace(
                    ".json", f"_{kwargs['prediction_name']}.json"
                )

            with open(qrels_save_path, "w") as f:
                json.dump(results, f)

        ndcg, _map, recall, precision, naucs = retriever.evaluate(
            relevant_docs,
            results,
            retriever.k_values,
            ignore_identical_ids=kwargs.get("ignore_identical_ids", True),
        )
        mrr, naucs = retriever.evaluate_custom(
            relevant_docs, results, retriever.k_values, "mrr"
        )
        scores = {
            **{f"ndcg_at_{k.split('@')[1]}": v for (k, v) in ndcg.items()},
            **{f"map_at_{k.split('@')[1]}": v for (k, v) in _map.items()},
            **{f"recall_at_{k.split('@')[1]}": v for (k, v) in recall.items()},
            **{f"precision_at_{k.split('@')[1]}": v for (k, v) in precision.items()},
            **{f"naucs_at_{k.split('@')[1]}": v for (k, v) in naucs.items()},
            **{f"mrr_at_{k.split('@')[1]}": v for (k, v) in mrr.items()},
        }
        return scores, results

    def create_qrel_diff(self, og_qrels, changed_qrels):
        newly_irrelevant_qrels = {}
        for qid in og_qrels:
            newly_irrelevant_qrels[qid] = []
            for doc_id in og_qrels[qid]:
                if changed_qrels[qid][doc_id] != og_qrels[qid][doc_id]:
                    newly_irrelevant_qrels[qid].append(doc_id)

        return newly_irrelevant_qrels

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> InstructionRetrievalDescriptiveStatistics:
        if hf_subset:
            corpus = self.corpus[hf_subset][split]
            queries = self.queries[hf_subset][split]
            relevant_docs = self.og_relevant_docs[hf_subset][split]
            og_instructions = self.og_instructions[hf_subset][split]
            changed_instructions = self.changed_instructions[hf_subset][split]
            top_ranked = self.top_ranked[hf_subset][split]
        elif compute_overall:
            corpus = {}
            queries = {}
            relevant_docs = {}
            og_instructions = {}
            changed_instructions = {}
            top_ranked = {}
            for hf_subset in self.metadata.eval_langs:
                corpus.update(process_docs(self.corpus, hf_subset, split))
                queries.update(process_docs(self.queries, hf_subset, split))
                relevant_docs.update(
                    process_relevant_docs(self.og_relevant_docs, hf_subset, split)
                )
                og_instructions.update(
                    process_docs(
                        self.og_instructions,
                        hf_subset,
                        split,
                    )
                )
                changed_instructions.update(
                    process_docs(self.changed_instructions, hf_subset, split)
                )
                top_ranked.update(process_top_ranked(self.top_ranked, hf_subset, split))
        else:
            corpus = self.corpus[split]
            queries = self.queries[split]
            relevant_docs = self.og_relevant_docs[split]
            og_instructions = self.og_instructions[split]
            changed_instructions = self.changed_instructions[split]
            top_ranked = self.top_ranked[split]

        corpus_combined = [
            doc.get("title", "") + doc["text"] for doc in corpus.values()
        ]
        corpus_len = [len(doc) for doc in corpus_combined]
        total_corpus_len = sum(corpus_len)

        queries_len = [len(query) for query in queries.values()]
        total_queries_len = sum(queries_len)
        instructions_len = [
            len(instruction) for instruction in og_instructions.values()
        ]
        total_instructions_len = sum(instructions_len)
        changed_instructions_len = [
            len(instruction) for instruction in changed_instructions.values()
        ]
        total_changed_instructions_len = sum(changed_instructions_len)
        qrels_non_zero = [
            sum(1 for doc_id in docs if docs[doc_id] != 0)
            for docs in relevant_docs.values()
        ]
        num_qrels_non_zero = sum(qrels_non_zero)
        qrels_per_doc = num_qrels_non_zero / len(relevant_docs) if len(queries) else 0
        ranked_per_query = [len(docs) for docs in top_ranked.values()]
        top_ranked_per_query = (
            sum(ranked_per_query) / len(queries) if len(queries) else 0
        )
        return InstructionRetrievalDescriptiveStatistics(
            num_samples=len(queries) + len(corpus),
            num_docs=len(corpus),
            num_queries=len(queries),
            number_of_characters=total_corpus_len
            + total_queries_len
            + total_instructions_len
            + total_changed_instructions_len,
            min_document_length=min(corpus_len),
            average_document_length=(
                total_corpus_len / len(corpus) if len(corpus) else 0
            ),
            max_document_length=max(corpus_len),
            unique_docs=len(set(corpus_combined)),
            min_query_length=min(queries_len),
            average_query_length=(
                total_queries_len / len(queries) if len(queries) else 0
            ),
            max_query_length=max(queries_len),
            unique_queries=len(set(queries.values())),
            min_instruction_length=min(instructions_len),
            average_instruction_length=(
                total_instructions_len / len(queries) if len(queries) else 0
            ),
            max_instruction_length=max(instructions_len),
            unique_instructions=len(set(og_instructions.values())),
            min_changed_instruction_length=min(changed_instructions_len),
            average_changed_instruction_length=(
                total_changed_instructions_len / len(queries) if len(queries) else 0
            ),
            max_changed_instruction_length=max(changed_instructions_len),
            unique_changed_instructions=len(set(changed_instructions.values())),
            min_average_relevant_docs_per_query=min(qrels_non_zero),
            average_relevant_docs_per_query=qrels_per_doc,
            max_average_relevant_docs_per_query=max(qrels_non_zero),
            min_average_top_ranked_per_query=min(ranked_per_query),
            average_top_ranked_per_query=top_ranked_per_query,
            max_average_top_ranked_per_query=max(ranked_per_query),
        )


def process_docs(
    collection: dict[str, dict[str, dict[str, str]]], hf_subset: str, split: str
) -> dict[str, str]:
    """Collections can contain overlapping ids in different splits. Prepend split to avoid this"""
    return {
        f"{split}_{hf_subset}_{k}": v for k, v in collection[hf_subset][split].items()
    }


def process_relevant_docs(
    collection: dict[str, dict[str, dict[str, dict[str, int]]]],
    hf_subset: str,
    split: str,
) -> dict[str, dict[str, int]]:
    """Collections can contain overlapping ids in different splits. Prepend split to avoid this"""
    return_collection = {}
    for query_id, relevant in collection[hf_subset][split].items():
        return_collection[f"{split}_{hf_subset}_{query_id}"] = {
            f"{split}_{hf_subset}_{doc_id}": value for doc_id, value in relevant.items()
        }
    return return_collection


def process_top_ranked(
    collection: dict[str, dict[str, dict[str, list[str]]]], hf_subset: str, split: str
) -> dict[str, list[str]]:
    return_collection = {}
    for query_id, docs_id in collection[hf_subset][split].items():
        return_collection[f"{split}_{hf_subset}_{query_id}"] = [
            f"{split}_{hf_subset}_{doc_id}" for doc_id in docs_id
        ]
    return return_collection



from __future__ import annotations

import pytest

import mteb
from mteb import get_task, get_tasks
from mteb.abstasks.AbsTask import AbsTask
from mteb.abstasks.TaskMetadata import TASK_DOMAIN, TASK_TYPE
from mteb.custom_validators import MODALITIES
from mteb.overview import MTEBTasks


def test_get_tasks_size_differences():
    assert len(get_tasks()) > 0
    assert len(get_tasks()) >= len(get_tasks(languages=["eng"]))
    assert len(get_tasks()) >= len(get_tasks(script=["Latn"]))
    assert len(get_tasks()) >= len(get_tasks(domains=["Legal"]))
    assert len(get_tasks()) >= len(get_tasks(languages=["eng", "deu"]))
    assert len(get_tasks()) >= len(get_tasks(modalities=["text"]))
    assert len(get_tasks()) >= len(get_tasks(modalities=["image"]))
    assert len(get_tasks(languages=["eng", "deu"])) >= len(
        get_tasks(languages=["eng", "deu"])
    )
    assert len(get_tasks(modalities=["text", "image"])) >= len(
        get_tasks(modalities=["text"])
    )
    assert len(get_tasks(modalities=["text", "image"])) >= len(
        get_tasks(modalities=["image"])
    )


@pytest.mark.parametrize(
    "task_name", ["BornholmBitextMining", "CQADupstackRetrieval", "Birdsnap"]
)
@pytest.mark.parametrize("eval_splits", [["test"], None])
@pytest.mark.parametrize("modalities", [["text"], None])
@pytest.mark.parametrize("exclusive_modality_filter", [True, False])
def test_get_task(
    task_name: str,
    eval_splits: list[str] | None,
    modalities: list[MODALITIES] | None,
    exclusive_modality_filter: bool,
):
    task = get_task(
        task_name,
        eval_splits=eval_splits,
        modalities=modalities,
        exclusive_modality_filter=exclusive_modality_filter,
    )
    assert isinstance(task, AbsTask)
    assert task.metadata.name == task_name
    if eval_splits:
        for split in task.eval_splits:
            assert split in eval_splits
    else:
        assert task.eval_splits == task.metadata.eval_splits

    if modalities:
        if task.modalities:
            if exclusive_modality_filter:
                # With exclusive filter, task modalities must exactly match the requested modalities
                assert set(task.modalities) == set(modalities)
            else:
                # With inclusive filter, task modalities must have overlap with requested modalities
                assert any(mod in task.modalities for mod in modalities)


@pytest.mark.parametrize("languages", [["eng", "deu"], ["eng"], None])
@pytest.mark.parametrize("script", [["Latn"], ["Cyrl"], None])
@pytest.mark.parametrize("domains", [["Legal"], ["Medical", "Non-fiction"], None])
@pytest.mark.parametrize("task_types", [["Classification"], ["Clustering"], None])
@pytest.mark.parametrize("exclude_superseded_datasets", [True, False])
@pytest.mark.parametrize("modalities", [["text"], ["image"], ["text", "image"], None])
@pytest.mark.parametrize("exclusive_modality_filter", [True, False])
@pytest.mark.parametrize("exclude_aggregate", [True, False])
def test_get_tasks(
    languages: list[str],
    script: list[str],
    domains: list[TASK_DOMAIN],
    task_types: list[TASK_TYPE] | None,
    exclude_superseded_datasets: bool,
    modalities: list[MODALITIES] | None,
    exclusive_modality_filter: bool,
    exclude_aggregate: bool,
):
    tasks = mteb.get_tasks(
        languages=languages,
        script=script,
        domains=domains,
        task_types=task_types,
        exclude_superseded=exclude_superseded_datasets,
        modalities=modalities,
        exclusive_modality_filter=exclusive_modality_filter,
        exclude_aggregate=exclude_aggregate,
    )

    for task in tasks:
        if languages:
            assert set(languages).intersection(task.metadata.languages)
        if script:
            assert set(script).intersection(task.metadata.scripts)
        if domains:
            task_domains = (
                set(task.metadata.domains) if task.metadata.domains else set()
            )
            assert set(domains).intersection(set(task_domains))
        if task_types:
            assert task.metadata.type in task_types
        if exclude_superseded_datasets:
            assert task.superseded_by is None
        if modalities:
            if exclusive_modality_filter:
                assert set(task.modalities) == set(modalities)
            else:
                assert any(mod in task.modalities for mod in modalities)
        if exclude_aggregate:
            # Aggregate tasks should be excluded
            assert not task.is_aggregate


def test_get_tasks_filtering():
    """Tests that get_tasks filters tasks for languages within the task, i.e. that a multilingual task returns only relevant subtasks for the
    specified languages
    """
    tasks = get_tasks(languages=["eng"])

    for task in tasks:
        if (
            task.is_multilingual
            and task.metadata.name != "STS17MultilingualVisualSTSEng"
        ):
            assert isinstance(task.metadata.eval_langs, dict)

            for hf_subset in task.hf_subsets:
                assert "eng-Latn" in task.metadata.eval_langs[hf_subset], (
                    f"{task.metadata.name}"
                )


@pytest.mark.parametrize("script", [["Latn"], ["Cyrl"], None])
@pytest.mark.parametrize("task_types", [["Classification"], ["Clustering"], None])
@pytest.mark.parametrize("modalities", [["text"], ["image"], None])
def test_MTEBTasks(
    script: list[str],
    task_types: list[TASK_TYPE] | None,
    modalities: list[MODALITIES] | None,
):
    tasks = mteb.get_tasks(script=script, task_types=task_types, modalities=modalities)
    assert isinstance(tasks, MTEBTasks)
    langs = tasks.languages
    for t in tasks:
        assert len(langs.intersection(t.languages)) > 0

    # check for header of a table
    n_langs = len(tasks)
    assert len(tasks.to_markdown().split("\n")) - 3 == n_langs


def test_all_tasks_fetch():
    """Test that all tasks can be fetched"""
    mteb.MTEB.mteb_tasks()


@pytest.mark.parametrize("modalities", [["text"], ["image"], ["text", "image"]])
def test_get_tasks_with_exclusive_modality_filter(modalities):
    """Test exclusive_modality_filter with actual tasks (if available)"""
    text_tasks_exclusive = get_tasks(
        modalities=modalities, exclusive_modality_filter=True
    )
    for task in text_tasks_exclusive:
        assert set(task.modalities) == set(modalities)



from __future__ import annotations

import re
from pathlib import Path
from typing import get_args

import polars as pl

import mteb
from mteb.abstasks.TaskMetadata import TASK_TYPE
from mteb.languages import ISO_TO_FAM_LEVEL0, ISO_TO_LANGUAGE, PROGRAMMING_LANGS


def author_from_bibtex(bibtex: str | None) -> str:
    """Create (Authors, Year) from bibtex entry (author = {Authors}, year = {Year})"""
    if bibtex is None:
        return ""
    # get authors from bibtex (author = {Authors} or author={Authors})
    authors = re.search(r"author\s*=\s*{([^}]*)}", bibtex)
    if authors is None:
        return ""
    authors = authors.group(1)
    authors = [a.split(", ") for a in authors.split(" and ")]
    author_str_w_et_al = (
        authors[0][0] + " et al." if len(authors[0]) > 1 else authors[0][0]
    )
    # replace any newline characters
    author_str_w_et_al = author_str_w_et_al.replace("\n", " ")
    year = re.search(r"year\s*=\s*{([^}]*)}", bibtex)
    if year is None:
        return ""
    year_str = year.group(1)
    return f" ({author_str_w_et_al}, {year_str})"


def round_floats_in_dict(d: dict, precision: int = 2) -> dict:
    if not isinstance(d, dict):
        return d
    for key, value in d.items():
        if isinstance(value, float):
            d[key] = round(value, precision)
        elif isinstance(value, dict):
            d[key] = round_floats_in_dict(value, precision)
    return d


def task_to_markdown_row(task: mteb.AbsTask) -> str:
    name = task.metadata.name
    name_w_reference = (
        f"[{name}]({task.metadata.reference})" if task.metadata.reference else name
    )
    domains = (
        "[" + ", ".join(sorted(task.metadata.domains)) + "]"
        if task.metadata.domains
        else ""
    )
    n_samples = task.metadata.n_samples
    dataset_statistics = round_floats_in_dict(task.metadata.descriptive_stats)
    name_w_reference += author_from_bibtex(task.metadata.bibtex_citation)

    return f"| {name_w_reference} | {task.metadata.languages} | {task.metadata.type} | {task.metadata.category} | {domains} | {n_samples} | {dataset_statistics} |"


def create_tasks_table(tasks: list[mteb.AbsTask]) -> str:
    table = """
| Name | Languages | Type | Category | Domains | # Samples | Dataset statistics |
|------|-----------|------|----------|---------|-----------|--------------------|
"""
    for task in tasks:
        table += task_to_markdown_row(task) + "\n"
    return table


def create_task_lang_table(tasks: list[mteb.AbsTask], sort_by_sum=False) -> str:
    table_dict = {}
    ## Group by language. If it is a multilingual dataset, 1 is added to all languages present.
    for task in tasks:
        for lang in task.metadata.languages:
            if lang in PROGRAMMING_LANGS:
                lang = "code"
            if table_dict.get(lang) is None:
                table_dict[lang] = {k: 0 for k in sorted(get_args(TASK_TYPE))}
            table_dict[lang][task.metadata.type] += 1

    ## Wrangle for polars
    pl_table_dict = []
    for lang, d in table_dict.items():
        d.update({"0-lang-code": lang})  # for sorting columns
        pl_table_dict.append(d)

    df = pl.DataFrame(pl_table_dict).sort(by="0-lang-code")
    df = df.with_columns(
        pl.col("0-lang-code")
        .replace_strict(ISO_TO_LANGUAGE, default="unknown")
        .alias("1-lang-name")
    )
    df = df.with_columns(
        pl.col("0-lang-code")
        .replace_strict(ISO_TO_FAM_LEVEL0, default="Unclassified")
        .alias("2-lang-fam")
    )

    df = df.with_columns(sum=pl.sum_horizontal(get_args(TASK_TYPE)))
    df = df.select(sorted(df.columns))
    if sort_by_sum:
        df = df.sort(by="sum", descending=True)

    total = df.sum()
    task_names = sorted(get_args(TASK_TYPE))
    headers = ["ISO Code", "Language", "Family"] + task_names + ["Sum"]
    table_header = "| " + " | ".join(headers) + " |"
    separator_line = "|"
    for header in headers:
        width = len(header) + 2
        separator_line += "-" * width + "|"
    table = table_header + "\n" + separator_line + "\n"

    for row in df.iter_rows():
        table += f"| {row[0]} "
        for num in row[1:]:
            table += f"| {num} "
        table += "|\n"

    for row in total.iter_rows():
        table += "| Total "
        for num in row[:-1]:
            table += f"| {num} "
        table += "|\n"

    return table


def insert_tables(
    file_path: str, tables: list[str], tags: list[str] = ["TASKS TABLE"]
) -> None:
    """Insert tables within <!-- TABLE START --> and <!-- TABLE END --> or similar tags."""
    md = Path(file_path).read_text(encoding="utf-8")

    for table, tag in zip(tables, tags):
        start = f"<!-- {tag} START -->"
        end = f"<!-- {tag} END -->"
        # Ensure a newline after the start tag
        md = md.replace(
            md[md.index(start) + len(start) : md.index(end)],
            f"\n{table}\n",
        )

    Path(file_path).write_text(md, encoding="utf-8")


def main():
    tasks = mteb.get_tasks()
    tasks = sorted(tasks, key=lambda x: x.metadata.name)

    tasks_table = create_tasks_table(tasks)
    task_lang_table = create_task_lang_table(tasks)

    file_path = Path(__file__).parent / "tasks.md"

    insert_tables(
        file_path,
        tables=[tasks_table, task_lang_table],
        tags=["TASKS TABLE", "TASK LANG TABLE"],
    )


if __name__ == "__main__":
    main()



from __future__ import annotations

import numpy as np
from datasets import DatasetDict, load_dataset

from mteb.abstasks.Image.AbsTaskImageTextPairClassification import (
    AbsTaskImageTextPairClassification,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class ImageCoDe(AbsTaskImageTextPairClassification):
    texts_column_names = ["text"]
    images_column_names = [
        "correct_image",
        "false_image_1",
        "false_image_2",
        "false_image_3",
        "false_image_4",
        "false_image_5",
        "false_image_6",
        "false_image_7",
        "false_image_8",
        "false_image_9",
    ]

    metadata = TaskMetadata(
        name="ImageCoDe",
        description="Identify the correct image from a set of similar images based on a precise caption.",
        reference="https://aclanthology.org/2022.acl-long.241.pdf",
        dataset={
            "path": "JamieSJS/imagecode-multi",
            "revision": "d28adfd8b34fefa546fdf94bdc352622b2575f6c",
        },
        type="Compositionality",
        category="it2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="image_acc",
        date=("2022-05-22", "2022-05-27"),  # conference dates
        domains=["Web", "Written"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation="""@article{krojer2022image,
  title={Image retrieval from contextual descriptions},
  author={Krojer, Benno and Adlakha, Vaibhav and Vineet, Vibhav and Goyal, Yash and Ponti, Edoardo and Reddy, Siva},
  journal={arXiv preprint arXiv:2203.15867},
  year={2022}
}""",
        descriptive_stats={
            "n_samples": {"test": 25322},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 0.0,
                    "num_documents": 23020,
                    "num_queries": 2302,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        corpus = load_dataset(
            self.metadata_dict["dataset"]["path"],
            "corpus",
            cache_dir=kwargs.get("cache_dir", None),
            revision=self.metadata_dict["dataset"]["revision"],
        )["corpus"]
        query = load_dataset(
            self.metadata_dict["dataset"]["path"],
            "query",
            cache_dir=kwargs.get("cache_dir", None),
            revision=self.metadata_dict["dataset"]["revision"],
        )["test"]
        qrels = load_dataset(
            self.metadata_dict["dataset"]["path"],
            "qrels",
            cache_dir=kwargs.get("cache_dir", None),
            revision=self.metadata_dict["dataset"]["revision"],
        )["test"]

        corpus_ids = corpus["id"]
        corpus_id_to_idx = {cid: idx for idx, cid in enumerate(corpus_ids)}

        def build_mappings(qrels):
            correct_answers = {}
            candidate_pools = {}
            for row in qrels:
                qid = row["query-id"]
                cid = row["corpus-id"]
                if qid not in candidate_pools:
                    candidate_pools[qid] = []
                candidate_pools[qid].append(cid)
                if row["score"] == 1:
                    correct_answers[qid] = cid
            return correct_answers, candidate_pools

        correct_answers, candidate_pools = build_mappings(qrels)

        def process_example(example):
            qid = example["id"]
            correct_id = correct_answers[qid]
            candidates = candidate_pools[qid]

            candidate_indices = [corpus_id_to_idx[cid] for cid in candidates]

            correct_mask = np.array([cid == correct_id for cid in candidates])
            correct_idx = candidate_indices[np.argmax(correct_mask)]
            incorrect_indices = [
                idx for idx, mask in zip(candidate_indices, ~correct_mask) if mask
            ]
            return {
                "text": example["text"],
                "correct_image": corpus[correct_idx]["image"],
                **{
                    f"false_image_{i + 1}": corpus[idx]["image"]
                    for i, idx in enumerate(incorrect_indices[:9])
                },
            }

        dataset = query.map(
            process_example, num_proc=4, remove_columns=query.column_names
        )

        self.dataset = DatasetDict({"test": dataset})
        self.dataset_transform()
        self.data_loaded = True



from __future__ import annotations

import random

import datasets

from mteb.abstasks.AbsTaskBitextMining import AbsTaskBitextMining
from mteb.abstasks.TaskMetadata import TaskMetadata

TEST_SAMPLES = 2048


class VieMedEVBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="VieMedEVBitextMining",
        dataset={
            "path": "nhuvo/MedEV",
            "revision": "d03c69413bc53d1cea5a5375b3a953c4fee35ecd",
            "trust_remote_code": True,
        },
        description="A high-quality Vietnamese-English parallel data from the medical domain for machine translation",
        reference="https://aclanthology.org/2015.iwslt-evaluation.11/",
        type="BitextMining",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn", "vie-Latn"],
        main_score="f1",
        date=("2024-08-28", "2022-03-28"),
        domains=["Medical", "Written"],
        task_subtypes=[],
        license="cc-by-nc-4.0",  # version is assumed, but was previously unspecified
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation="""@inproceedings{medev,
    title     = {{Improving Vietnamese-English Medical Machine Translation}},
    author    = {Nhu Vo and Dat Quoc Nguyen and Dung D. Le and Massimo Piccardi and Wray Buntine},
    booktitle = {Proceedings of the 2024 Joint International Conference on Computational Linguistics, Language Resources and Evaluation (LREC-COLING)},
    year      = {2024}
}""",
    )

    def dataset_transform(self):
        # Convert to standard format
        ds = {}
        seed = 42
        random.seed(seed)
        # Get all texts
        all_texts = self.dataset["test"]["text"]

        # Determine the midpoint of the list
        mid_index = len(all_texts) // 2
        # Pairs are in two halves
        en_sentences = all_texts[:mid_index]
        vie_sentences = all_texts[mid_index:]
        assert len(en_sentences) == len(vie_sentences), (
            "The split does not result in equal halves."
        )

        # Downsample
        indices = list(range(len(en_sentences)))
        random.shuffle(indices)
        sample_indices = indices[:TEST_SAMPLES]
        en_sentences = [en_sentences[i] for i in sample_indices]
        vie_sentences = [vie_sentences[i] for i in sample_indices]
        assert len(en_sentences) == len(vie_sentences) == TEST_SAMPLES, (
            f"Exceeded {TEST_SAMPLES} samples for 'test' split."
        )

        # Return dataset
        ds["test"] = datasets.Dataset.from_dict(
            {"sentence1": vie_sentences, "sentence2": en_sentences}
        )
        self.dataset = datasets.DatasetDict(ds)



from __future__ import annotations

import pytest

from mteb import MTEB
from tests.test_benchmark.mock_models import (
    MockSentenceTransformer,
)
from tests.test_benchmark.mock_tasks import (
    MockMultilingualRetrievalTask,
    MockMultilingualSTSTask,
    MockRetrievalTask,
)


@pytest.fixture
def model():
    return MockSentenceTransformer()


@pytest.fixture
def tasks():
    return [MockRetrievalTask()]


@pytest.fixture
def multilingual_tasks():
    return [MockMultilingualRetrievalTask()]


def test_all_splits_evaluated(model, tasks, tmp_path):
    evaluation = MTEB(tasks=tasks)
    results = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "all_splits_evaluated"),
        verbosity=2,
    )

    assert "MockRetrievalTask" == results[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockRetrievalTask"]) == {"val", "test"}
    assert len(last_evaluated_splits["MockRetrievalTask"]) == 2
    assert results[0].scores.keys() == {"val", "test"}


def test_one_missing_split(model, tasks, tmp_path):
    evaluation = MTEB(tasks=tasks)
    results = evaluation.run(
        model,
        eval_splits=["val"],
        output_folder=str(tmp_path / "testcase2"),
        verbosity=2,
    )

    assert "MockRetrievalTask" == results[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockRetrievalTask"]) == {"val"}
    assert len(last_evaluated_splits["MockRetrievalTask"]) == 1
    assert results[0].scores.keys() == {"val"}

    results2 = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "testcase2"),
        verbosity=2,
    )

    assert "MockRetrievalTask" == results2[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockRetrievalTask"]) == {"test"}
    assert len(last_evaluated_splits["MockRetrievalTask"]) == 1
    assert results2[0].scores.keys() == {"test", "val"}


def test_no_missing_splits(model, tasks, tmp_path):
    evaluation = MTEB(tasks=tasks)
    results = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "testcase3"),
        verbosity=2,
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert "MockRetrievalTask" in last_evaluated_splits
    assert len(last_evaluated_splits["MockRetrievalTask"]) == 2
    assert results[0].scores.keys() == {"test", "val"}

    evaluation = MTEB(tasks=tasks)
    results = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "testcase3"),
        verbosity=2,
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits) == 0
    assert results[0].scores.keys() == {"test", "val"}


def test_all_languages_evaluated(model, multilingual_tasks, tmp_path):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "all_lang_evaluated"),
        verbosity=2,
        eval_subsets=None,
    )
    assert "MockMultilingualRetrievalTask" == results[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert "MockMultilingualRetrievalTask" in last_evaluated_splits
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert last_evaluated_splits["MockMultilingualRetrievalTask"] == ["test"]
    assert results[0].scores.keys() == {"test"}
    assert len(results[0].scores["test"]) == 2


def test_missing_language(model, multilingual_tasks, tmp_path):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "missing_lang_test"),
        verbosity=2,
        eval_subsets=["eng"],
    )

    assert "MockMultilingualRetrievalTask" == results[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert "MockMultilingualRetrievalTask" in last_evaluated_splits
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert last_evaluated_splits["MockMultilingualRetrievalTask"] == ["test"]
    assert results[0].scores.keys() == {"test"}
    assert results[0].languages == ["eng"]

    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "missing_lang_test"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert last_evaluated_splits["MockMultilingualRetrievalTask"] == ["test"]
    assert sorted(results[0].languages) == ["eng", "fra"]
    assert results[0].scores.keys() == {"test"}
    assert len(results[0].scores["test"]) == 2


def test_no_missing_languages(model, multilingual_tasks, tmp_path):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "no_missing_lang_test"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert "MockMultilingualRetrievalTask" in last_evaluated_splits
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert results[0].scores.keys() == {"test"}
    assert len(results[0].scores["test"]) == 2
    assert sorted(results[0].languages) == ["eng", "fra"]

    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "no_missing_lang_test"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits) == 0
    assert results[0].scores.keys() == {"test"}
    assert len(results[0].scores["test"]) == 2
    assert sorted(results[0].languages) == ["eng", "fra"]


def test_partial_languages(model, multilingual_tasks, tmp_path):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "partial_lang_test"),
        verbosity=2,
        eval_subsets=["fra"],
    )
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert last_evaluated_splits["MockMultilingualRetrievalTask"] == ["test"]
    assert results[0].scores.keys() == {"test"}
    assert len(results[0].scores["test"]) == 1
    assert results[0].languages == ["fra"]

    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "partial_lang_test"),
        verbosity=2,
        eval_subsets=["fra", "eng"],
    )
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert last_evaluated_splits["MockMultilingualRetrievalTask"] == ["test"]
    assert results[0].scores.keys() == {"test"}
    assert len(results[0].scores["test"]) == 2
    assert sorted(results[0].languages) == ["eng", "fra"]


def test_multilingual_one_missing_split_no_missing_lang(
    model, multilingual_tasks, tmp_path
):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["val"],
        output_folder=str(tmp_path / "partial_langs_partial_splits"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert set(last_evaluated_splits["MockMultilingualRetrievalTask"]) == {"val"}
    assert sorted(results[0].languages) == ["eng", "fra"]
    assert results[0].scores.keys() == {"val"}
    assert len(results[0].scores["val"]) == 2

    results = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "partial_langs_partial_splits"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockMultilingualRetrievalTask"]) == {"test"}
    assert sorted(results[0].languages) == ["eng", "fra"]
    assert results[0].scores.keys() == {"test", "val"}
    assert len(results[0].scores["test"]) == 2
    assert len(results[0].scores["val"]) == 2


def test_multilingual_one_missing_lang_in_one_split(
    model, multilingual_tasks, tmp_path
):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=["val"],
        output_folder=str(tmp_path / "one_lang_one_split"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockMultilingualRetrievalTask"]) == {"val"}
    assert sorted(results[0].languages) == ["eng", "fra"]
    assert results[0].scores.keys() == {"val"}
    assert len(results[0].scores["val"]) == 2

    results = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "one_lang_one_split"),
        verbosity=2,
        eval_subsets=["eng"],
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockMultilingualRetrievalTask"]) == {"test"}
    assert sorted(results[0].languages) == ["eng", "fra"]
    assert results[0].scores.keys() == {"test", "val"}
    assert len(results[0].scores["test"]) == 1
    assert len(results[0].scores["val"]) == 2

    results = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "one_lang_one_split"),
        verbosity=2,
        eval_subsets=["eng", "fra"],
    )

    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert set(last_evaluated_splits["MockMultilingualRetrievalTask"]) == {"test"}
    assert sorted(results[0].languages) == ["eng", "fra"]
    # output merged result with previous results
    assert results[0].scores.keys() == {"test", "val"}
    assert len(results[0].scores["test"]) == 2


def test_all_splits_evaluated_with_overwrite(model, tasks, tmp_path):
    evaluation = MTEB(tasks=tasks)
    results = evaluation.run(
        model,
        eval_splits=["val"],
        output_folder=str(tmp_path / "all_splits_evaluated_with_overwrite"),
        verbosity=2,
    )

    assert "MockRetrievalTask" == results[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits["MockRetrievalTask"]) == 1
    assert set(last_evaluated_splits["MockRetrievalTask"]) == {"val"}
    assert results[0].scores.keys() == {"val"}

    results2 = evaluation.run(
        model,
        eval_splits=["val", "test"],
        output_folder=str(tmp_path / "all_splits_evaluated_with_overwrite"),
        verbosity=2,
        overwrite_results=True,
    )
    assert "MockRetrievalTask" == results2[0].task_name
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert len(last_evaluated_splits["MockRetrievalTask"]) == 2
    assert set(last_evaluated_splits["MockRetrievalTask"]) == {"val", "test"}
    assert results2[0].scores.keys() == {"val", "test"}


def test_all_splits_subsets_evaluated_with_overwrite(
    model, multilingual_tasks, tmp_path
):
    evaluation = MTEB(tasks=multilingual_tasks)
    results = evaluation.run(
        model,
        eval_splits=[
            "test",
        ],
        output_folder=str(tmp_path / "all_splits_subsets_evaluated_with_overwrite"),
        verbosity=2,
        eval_subsets=["fra"],
    )
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert "MockMultilingualRetrievalTask" in last_evaluated_splits
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert results[0].scores.keys() == {"test"}
    for split in ["test"]:
        assert len(results[0].scores[split]) == 1
        assert sorted(results[0].languages) == ["fra"]

    results2 = evaluation.run(
        model,
        eval_splits=["test"],
        output_folder=str(tmp_path / "all_splits_subsets_evaluated_with_overwrite"),
        verbosity=2,
        eval_subsets=["fra", "eng"],
        overwrite_results=True,
    )
    last_evaluated_splits = evaluation.get_last_evaluated_splits()
    assert "MockMultilingualRetrievalTask" in last_evaluated_splits
    assert len(last_evaluated_splits["MockMultilingualRetrievalTask"]) == 1
    assert results2[0].scores.keys() == {"test"}
    for split in ["test"]:
        assert len(results2[0].scores[split]) == 2
        assert sorted(results2[0].languages) == ["eng", "fra"]


def test_splits_evaluated_with_prefiltering():
    """Test that the evaluation only runs on the specified languages. Issue https://github.com/embeddings-benchmark/mteb/pull/1787#issuecomment-2598205049"""
    task = MockMultilingualSTSTask().filter_languages(languages=["fra"])

    evaluation = MTEB(tasks=[task])

    results = evaluation.run(MockSentenceTransformer(), overwrite_results=True)
    result_scores = results[0].scores

    assert len(result_scores) == 1
    assert "test" in result_scores
    assert len(result_scores["test"]) == 1
    assert result_scores["test"][0]["hf_subset"] == "fra"



from __future__ import annotations

from mteb.abstasks import AbsTask
from mteb.abstasks.aggregated_task import AbsTaskAggregate, AggregateTaskMetadata
from mteb.tasks.Retrieval import (
    CQADupstackAndroidNLRetrieval,
    CQADupstackEnglishNLRetrieval,
    CQADupstackGamingNLRetrieval,
    CQADupstackGisNLRetrieval,
    CQADupstackMathematicaNLRetrieval,
    CQADupstackPhysicsNLRetrieval,
    CQADupstackProgrammersNLRetrieval,
    CQADupstackStatsNLRetrieval,
    CQADupstackTexNLRetrieval,
    CQADupstackUnixNLRetrieval,
    CQADupstackWebmastersNLRetrieval,
    CQADupstackWordpressNLRetrieval,
)

task_list_cqa: list[AbsTask] = [
    CQADupstackAndroidNLRetrieval(),
    CQADupstackEnglishNLRetrieval(),
    CQADupstackGamingNLRetrieval(),
    CQADupstackGisNLRetrieval(),
    CQADupstackMathematicaNLRetrieval(),
    CQADupstackPhysicsNLRetrieval(),
    CQADupstackProgrammersNLRetrieval(),
    CQADupstackStatsNLRetrieval(),
    CQADupstackTexNLRetrieval(),
    CQADupstackUnixNLRetrieval(),
    CQADupstackWebmastersNLRetrieval(),
    CQADupstackWordpressNLRetrieval(),
]


class CQADupstackNLRetrieval(AbsTaskAggregate):
    metadata = AggregateTaskMetadata(
        name="CQADupstack-NL",
        description="CQADupStack: A Benchmark Data Set for Community Question-Answering Research. This a "
        "Dutch-translated version.",
        reference="https://huggingface.co/datasets/clips/beir-nl-cqadupstack",
        tasks=task_list_cqa,
        main_score="ndcg_at_10",
        type="Retrieval",  # since everything is retrieval - otherwise it would be "Aggregated"
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        date=("2015-12-01", "2015-12-01"),  # best guess: based on publication date
        domains=["Written", "Non-fiction"],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[""],
        sample_creation="machine-translated and verified",  # manually checked a small subset
        bibtex_citation="""@misc{banar2024beirnlzeroshotinformationretrieval,
    title={BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language},
     author={Nikolay Banar and Ehsan Lotfi and Walter Daelemans},
     year={2024},
     eprint={2412.08329},
     archivePrefix={arXiv},
     primaryClass={cs.CL},
     url={https://arxiv.org/abs/2412.08329},
}""",
        adapted_from=["CQADupstackRetrieval"],
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskPairClassification import AbsTaskPairClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class KlueNLI(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="KLUE-NLI",
        dataset={
            "path": "klue/klue",
            "name": "nli",
            "revision": "349481ec73fff722f88e0453ca05c77a447d967c",
        },
        description="Textual Entailment between a hypothesis sentence and a premise sentence. Part of the Korean Language Understanding Evaluation (KLUE).",
        reference="https://arxiv.org/abs/2105.09680",
        type="PairClassification",
        category="s2s",
        modalities=["text"],
        eval_splits=["validation"],
        eval_langs=["kor-Hang"],
        main_score="max_ap",
        date=("2016-01-01", "2020-12-31"),
        domains=["News", "Encyclopaedic", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{park2021klue,
      title={KLUE: Korean Language Understanding Evaluation},
      author={Sungjoon Park and Jihyung Moon and Sungdong Kim and Won Ik Cho and Jiyoon Han and Jangwon Park and Chisung Song and Junseong Kim and Yongsook Song and Taehwan Oh and Joohong Lee and Juhyun Oh and Sungwon Lyu and Younghoon Jeong and Inkwon Lee and Sangwoo Seo and Dongjun Lee and Hyunwoo Kim and Myeonghwa Lee and Seongbo Jang and Seungwon Do and Sunkyoung Kim and Kyungtae Lim and Jongwon Lee and Kyumin Park and Jamin Shin and Seonghyun Kim and Lucy Park and Alice Oh and Jungwoo Ha and Kyunghyun Cho},
      year={2021},
      eprint={2105.09680},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",  # 3000 - neutral samples
    )

    def dataset_transform(self):
        _dataset = {}
        for split in self.metadata.eval_splits:
            # keep labels 0=entailment and 2=contradiction, and map them as 1 and 0 for binary classification
            hf_dataset = self.dataset[split].filter(lambda x: x["label"] in [0, 2])
            hf_dataset = hf_dataset.map(
                lambda example: {"label": 0 if example["label"] == 2 else 1}
            )
            _dataset[split] = [
                {
                    "sentence1": hf_dataset["premise"],
                    "sentence2": hf_dataset["hypothesis"],
                    "labels": hf_dataset["label"],
                }
            ]
        self.dataset = _dataset



from __future__ import annotations

import pytest

import mteb
from mteb import ModelMeta


@pytest.mark.parametrize(
    ("model_name", "expected_memory"),
    [
        ("intfloat/e5-mistral-7b-instruct", 13563),  # multiple safetensors
        ("NovaSearch/jasper_en_vision_language_v1", 3802),  # bf16
        ("intfloat/multilingual-e5-small", 449),  # safetensors
        ("BAAI/bge-m3", 2167),  # pytorch_model.bin
    ],
)
def test_model_memory_usage(model_name: str, expected_memory: int | None):
    meta = mteb.get_model_meta(model_name)
    assert meta.memory_usage_mb is not None
    used_memory = round(meta.memory_usage_mb)
    assert used_memory == expected_memory


def test_model_memory_usage_api_model():
    meta = mteb.get_model_meta("openai/text-embedding-3-large")
    assert meta.memory_usage_mb is None


@pytest.mark.parametrize(
    "training_datasets",
    [
        {"Touche2020": []},  # parent task
        {"Touche2020-NL": []},  # child task
    ],
)
def test_model_similar_tasks(training_datasets):
    dummy_model_meta = ModelMeta(
        name="test/test_model",
        revision="test",
        release_date=None,
        languages=None,
        loader=None,
        n_parameters=None,
        memory_usage_mb=None,
        max_tokens=None,
        embed_dim=None,
        license=None,
        open_weights=None,
        public_training_code=None,
        public_training_data=None,
        framework=[],
        reference=None,
        similarity_fn_name=None,
        use_instructions=None,
        training_datasets=training_datasets,
        adapted_from=None,
        superseded_by=None,
    )
    expected = [
        "NanoTouche2020Retrieval",
        "Touche2020",
        "Touche2020-Fa",
        "Touche2020-NL",
        "Touche2020Retrieval.v3",
    ]
    assert sorted(dummy_model_meta.get_training_datasets().keys()) == expected


def test_model_name_without_prefix():
    with pytest.raises(ValueError):
        ModelMeta(
            name="test_model",
            revision="test",
            release_date=None,
            languages=None,
            loader=None,
            n_parameters=None,
            memory_usage_mb=None,
            max_tokens=None,
            embed_dim=None,
            license=None,
            open_weights=None,
            public_training_code=None,
            public_training_data=None,
            framework=[],
            reference=None,
            similarity_fn_name=None,
            use_instructions=None,
            training_datasets=None,
            adapted_from=None,
            superseded_by=None,
        )


def test_model_training_dataset_adapted():
    model_meta = mteb.get_model_meta("deepvk/USER-bge-m3")
    assert model_meta.adapted_from == "BAAI/bge-m3"
    # MIRACLRetrieval not in training_datasets of deepvk/USER-bge-m3, but in
    # training_datasets of BAAI/bge-m3
    assert "MIRACLRetrieval" in model_meta.get_training_datasets()



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class FrenkHrClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FrenkHrClassification",
        description="Croatian subset of the FRENK dataset",
        dataset={
            "path": "classla/FRENK-hate-hr",
            "revision": "e7fc9f3d8d6c5640a26679d8a50b1666b02cc41f",
            "trust_remote_code": True,
        },
        reference="https://arxiv.org/abs/1906.02045",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["hrv-Latn"],
        main_score="accuracy",
        date=("2021-05-28", "2021-05-28"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{ljubešić2019frenk,
        title={The FRENK Datasets of Socially Unacceptable Discourse in Slovene and English},
        author={Nikola Ljubešić and Darja Fišer and Tomaž Erjavec},
        year={2019},
        eprint={1906.02045},
        archivePrefix={arXiv},
        primaryClass={cs.CL},
        url={https://arxiv.org/abs/1906.02045}
        }""",
    )



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class StanfordCarsI2I(AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="StanfordCarsI2IRetrieval",
        description="Retrieve car images from 196 makes.",
        reference="https://pure.mpg.de/rest/items/item_2029263/component/file_2029262/content",
        dataset={
            "path": "isaacchung/stanford_cars_retrieval",
            "revision": "b27a0612211af3598bd11fe28af20928f20cce06",
        },
        type="Any2AnyRetrieval",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cv_recall_at_1",
        date=("2012-01-01", "2013-04-01"),
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation="""@inproceedings{Krause2013CollectingAL,
        title={Collecting a Large-scale Dataset of Fine-grained Cars},
        author={Jonathan Krause and Jia Deng and Michael Stark and Li Fei-Fei},
        year={2013},
        url={https://api.semanticscholar.org/CorpusID:16632981}
        }""",
        descriptive_stats={
            "n_samples": {"default": 8041},
            "avg_character_length": {
                "test": {
                    "average_document_length": 1074.894348894349,
                    "average_query_length": 77.06142506142506,
                    "num_documents": 8041,
                    "num_queries": 8041,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )
    skip_first_result = True



"""Tests for the BenchmarkResult class"""

from __future__ import annotations

import os
from pathlib import Path

import pandas as pd
import pytest

import mteb
from mteb.load_results import ModelResult
from mteb.load_results.benchmark_results import BenchmarkResults


@pytest.fixture
def benchmark_results() -> BenchmarkResults:
    tests_path = Path(__file__).parent.parent / "mock_mteb_cache"

    os.environ["MTEB_CACHE"] = str(tests_path)

    results = mteb.load_results(download_latest=False)

    return results


def test_indexing(benchmark_results: BenchmarkResults) -> None:
    model_res = benchmark_results.model_results[0]
    assert isinstance(model_res, ModelResult), (
        "indexing into the list should return a ModelResult"
    )


def test_select_models(benchmark_results: BenchmarkResults) -> None:
    model_name = "sentence-transformers/all-MiniLM-L6-v2"
    bench_res = benchmark_results.select_models([model_name])
    assert isinstance(bench_res, BenchmarkResults)
    assert isinstance(bench_res[0], ModelResult)
    assert len(bench_res.model_results) > 1  # multiple revisions
    assert bench_res[0].model_name == model_name

    # with revision
    model_meta = mteb.get_model_meta(model_name)
    bench_res = benchmark_results.select_models(
        names=[model_name],
        revisions=[model_meta.revision],
    )
    assert bench_res[0].model_name == model_name
    assert bench_res[0].model_revision == model_meta.revision
    assert len(bench_res.model_results) == 1  # only one revision

    # with model_meta
    model_meta = mteb.get_model_meta(model_name)
    bench_res = benchmark_results.select_models(
        names=[model_meta],
    )
    assert bench_res[0].model_name == model_name
    assert bench_res[0].model_revision == model_meta.revision
    assert len(bench_res.model_results) == 1  # only one revision


def test_select_tasks(benchmark_results: BenchmarkResults) -> None:
    tasks = [mteb.get_task("STS12")]
    bench_res = benchmark_results.select_tasks(tasks=tasks)
    task_names = bench_res.task_names
    assert isinstance(task_names, list)
    assert len(task_names) == 1
    assert task_names[0] == "STS12"


def test_join_revisions(benchmark_results: BenchmarkResults) -> None:
    model_name = "sentence-transformers/all-MiniLM-L6-v2"
    bench_res = benchmark_results.select_models([model_name])

    assert len(bench_res.model_revisions) == 2, (
        "There should only be two revisions for this model in the mock cache"
    )

    bench_res = bench_res.join_revisions()
    assert isinstance(bench_res, BenchmarkResults)
    assert len(bench_res.model_revisions) == 1
    revision = bench_res.model_revisions[0]["revision"]
    assert revision == mteb.get_model_meta(model_name).revision


def test_to_dataframe(
    benchmark_results: BenchmarkResults,
) -> None:
    required_columns = [
        "model_name",
        "task_name",
        "task_name",
        "score",
        "subset",
        "split",
    ]
    t1 = benchmark_results.to_dataframe(aggregation_level="subset", format="long")
    assert isinstance(t1, pd.DataFrame)
    assert all(col in t1.columns for col in required_columns), "Columns are missing"
    assert t1.shape[0] > 0, "Results table is empty"

    t2 = benchmark_results.to_dataframe(aggregation_level="split", format="long")
    assert all(
        col in t2.columns for col in required_columns if col not in ["subset"]
    ), "Columns are missing"
    assert "subset" not in t2.columns, "Subset column should not be present"
    assert t1.shape[0] >= t2.shape[0], (
        "Aggregation level 'split' should have more rows than 'subset'"
    )

    t3 = benchmark_results.to_dataframe(aggregation_level="task", format="long")
    assert all(
        col in t3.columns for col in required_columns if col not in ["subset", "split"]
    ), "Columns are missing"
    assert "subset" not in t3.columns, "Subset column should not be present"
    assert "split" not in t3.columns, "Split column should not be present"
    assert t2.shape[0] >= t3.shape[0], (
        "Aggregation level 'task' should have more rows than 'split'"
    )

    # test no model revisions
    benchmark_res = benchmark_results.join_revisions()
    t1 = benchmark_res.to_dataframe(aggregation_level="subset", format="long")
    assert "model_revision" not in t1.columns, (
        "Model revision column should not be present"
    )
    # Test the wide format
    model_name = "sentence-transformers/all-MiniLM-L6-v2"
    task = mteb.get_task("BornholmBitextMining")

    # simplify down to one model and one task
    br = benchmark_res.select_models([model_name]).select_tasks([task])

    t4_wide = br.to_dataframe(aggregation_level="task", format="wide")
    t4_long = br.to_dataframe(aggregation_level="task", format="long")
    assert isinstance(t4_wide, pd.DataFrame)

    # check that the scores are the same for a given model
    assert t4_wide[model_name][0] == t4_long["score"][0], (
        "The scores in wide and long format should be the same"
    )


def test_utility_properties(
    benchmark_results: BenchmarkResults,
) -> None:
    br = benchmark_results
    assert isinstance(br.task_names, list) and isinstance(br.task_names[0], str)
    assert (
        isinstance(br.languages, list)
        and isinstance(br.languages[0], str)
        and "eng" in br.languages
    )
    assert isinstance(br.model_names, list) and isinstance(br.model_names[0], str)
    assert (
        isinstance(br.model_revisions, list)
        and isinstance(br.model_revisions[0], dict)
        and "model_name" in br.model_revisions[0]
        and "revision" in br.model_revisions[0]
    )
    assert isinstance(br.task_types, list) and isinstance(br.task_types[0], str)
    assert isinstance(br.domains, list) and isinstance(br.domains[0], str)



from __future__ import annotations

from mteb.abstasks.AbsTaskPairClassification import AbsTaskPairClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class ArmenianParaphrasePC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="ArmenianParaphrasePC",
        description="asparius/Armenian-Paraphrase-PC",
        reference="https://github.com/ivannikov-lab/arpa-paraphrase-corpus",
        dataset={
            "path": "asparius/Armenian-Paraphrase-PC",
            "revision": "f43b4f32987048043a8b31e5e26be4d360c2438f",
        },
        type="PairClassification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["hye-Armn"],
        main_score="max_ap",
        date=("2021-01-01", "2022-04-06"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
        @misc{malajyan2020arpa,
            title={ARPA: Armenian Paraphrase Detection Corpus and Models},
            author={Arthur Malajyan and Karen Avetisyan and Tsolak Ghukasyan},
            year={2020},
            eprint={2009.12615},
            archivePrefix={arXiv},
            primaryClass={cs.CL}
        }
        """,
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("sent1", "sentence1")
        self.dataset = self.dataset.rename_column("sent2", "sentence2")



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class SketchyI2IRetrieval(AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="SketchyI2IRetrieval",
        description="Retrieve photos from sketches.",
        reference="https://arxiv.org/abs/2202.01747",
        dataset={
            "path": "JamieSJS/sketchy",
            "revision": "c8b8c1b7a2f0a92f1bfaaa1c9afc22aa42c61d5b",
        },
        type="Any2AnyRetrieval",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cv_recall_at_1",
        date=("2021-12-06", "2021-12-14"),  # conference dates
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation="""@inproceedings{ypsilantis2021met,
  title={The met dataset: Instance-level recognition for artworks},
  author={Ypsilantis, Nikolaos-Antonios and Garcia, Noa and Han, Guangxing and Ibrahimi, Sarah and Van Noord, Nanne and Tolias, Giorgos},
  booktitle={Thirty-fifth Conference on Neural Information Processing Systems Datasets and Benchmarks Track (Round 2)},
  year={2021}
}""",
        descriptive_stats={
            "n_samples": {"test": 452886},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 7.24,
                    "num_documents": 25000,
                    "num_queries": 452886,
                    "average_relevant_docs_per_query": 3623.0,
                }
            },
        },
    )
    skip_first_result = False



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class HeadlineClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="HeadlineClassification",
        dataset={
            "path": "ai-forever/headline-classification",
            "revision": "2fe05ee6b5832cda29f2ef7aaad7b7fe6a3609eb",
        },
        description="Headline rubric classification based on the paraphraser plus dataset.",
        reference="https://aclanthology.org/2020.ngt-1.6/",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="accuracy",
        date=("2009-01-01", "2020-01-01"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{gudkov-etal-2020-automatically,
        title = "Automatically Ranked {R}ussian Paraphrase Corpus for Text Generation",
        author = "Gudkov, Vadim  and
        Mitrofanova, Olga  and
        Filippskikh, Elizaveta",
        editor = "Birch, Alexandra  and
        Finch, Andrew  and
        Hayashi, Hiroaki  and
        Heafield, Kenneth  and
        Junczys-Dowmunt, Marcin  and
        Konstas, Ioannis  and
        Li, Xian  and
        Neubig, Graham  and
        Oda, Yusuke",
        booktitle = "Proceedings of the Fourth Workshop on Neural Generation and Translation",
        month = jul,
        year = "2020",
        address = "Online",
        publisher = "Association for Computational Linguistics",
        url = "https://aclanthology.org/2020.ngt-1.6",
        doi = "10.18653/v1/2020.ngt-1.6",
        pages = "54--59",
        abstract = "The article is focused on automatic development and ranking of a large corpus for Russian paraphrase generation which proves to be the first corpus of such type in Russian computational linguistics. Existing manually annotated paraphrase datasets for Russian are limited to small-sized ParaPhraser corpus and ParaPlag which are suitable for a set of NLP tasks, such as paraphrase and plagiarism detection, sentence similarity and relatedness estimation, etc. Due to size restrictions, these datasets can hardly be applied in end-to-end text generation solutions. Meanwhile, paraphrase generation requires a large amount of training data. In our study we propose a solution to the problem: we collect, rank and evaluate a new publicly available headline paraphrase corpus (ParaPhraser Plus), and then perform text generation experiments with manual evaluation on automatically ranked corpora using the Universal Transformer architecture.",
        }""",
        prompt="Classify the topic or theme of the given news headline",
    )

    def dataset_transform(self):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, n_samples=2048, splits=["test"]
        )



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class WebQAT2ITRetrieval(AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="WebQAT2ITRetrieval",
        description="Retrieve sources of information based on questions.",
        reference="https://openaccess.thecvf.com/content/CVPR2022/html/Chang_WebQA_Multihop_and_Multimodal_QA_CVPR_2022_paper.html",
        dataset={
            "path": "MRBench/mbeir_webqa_task2",
            "revision": "53db4c9f9c93cb74926a1c9d04dea7d7acac2f21",
        },
        type="Any2AnyRetrieval",
        category="t2it",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2022-01-01", "2022-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation="""@inproceedings{chang2022webqa,
      title={Webqa: Multihop and multimodal qa},
      author={Chang, Yingshan and Narang, Mridu and Suzuki, Hisami and Cao, Guihong and Gao, Jianfeng and Bisk, Yonatan},
      booktitle={Proceedings of the IEEE/CVF conference on computer vision and pattern recognition},
      pages={16495--16504},
       year={2022}
      }""",
        prompt={"query": "Find a Wikipedia image that answers this question."},
        descriptive_stats={
            "n_samples": {"test": 2511},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 0.0,
                    "num_documents": 403196,
                    "num_queries": 2511,
                    "average_relevant_docs_per_query": 1.4,
                }
            },
        },
    )



from __future__ import annotations

import logging
from unittest.mock import Mock, patch

import huggingface_hub
import pytest

import mteb
from mteb import MTEB
from mteb.abstasks import AbsTask
from mteb.abstasks.AbsTaskInstructionRetrieval import AbsTaskInstructionRetrieval
from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.AbsTaskSpeedTask import AbsTaskSpeedTask
from mteb.abstasks.aggregated_task import AbsTaskAggregate
from mteb.abstasks.Image.AbsTaskAny2AnyMultiChoice import AbsTaskAny2AnyMultiChoice
from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.Image.AbsTaskImageTextPairClassification import (
    AbsTaskImageTextPairClassification,
)
from mteb.abstasks.MultiSubsetLoader import MultiSubsetLoader
from mteb.overview import TASKS_REGISTRY

from ..test_benchmark.task_grid import (
    MOCK_MIEB_TASK_GRID_AS_STRING,
    MOCK_TASK_TEST_GRID_AS_STRING,
)

logging.basicConfig(level=logging.INFO)

ALL_MOCK_TASKS = MOCK_TASK_TEST_GRID_AS_STRING + MOCK_MIEB_TASK_GRID_AS_STRING

tasks = [t for t in MTEB().tasks_cls if t.metadata.name not in ALL_MOCK_TASKS]

datasets_not_available = [
    "AfriSentiLangClassification",
    "SNLHierarchicalClusteringP2P",
    "SNLClustering",
    "SNLHierarchicalClusteringS2S",
    "SNLRetrieval",
]


dataset_revisions = list(
    {  # deduplicate as multiple tasks rely on the same dataset (save us at least 100 test cases)
        (t.metadata.dataset["path"], t.metadata.dataset["revision"])
        for t in mteb.get_tasks(exclude_superseded=False)
        if not isinstance(t, (AbsTaskAggregate, AbsTaskSpeedTask))
        and t.metadata.name not in datasets_not_available
        and t.metadata.name not in ALL_MOCK_TASKS
    }
)


@pytest.mark.parametrize("task", tasks)
@patch("datasets.load_dataset")
@patch("datasets.concatenate_datasets")
def test_load_data(
    mock_concatenate_datasets: Mock, mock_load_dataset: Mock, task: AbsTask
):
    # TODO: We skip because this load_data is completely different.
    if (
        isinstance(task, AbsTaskRetrieval)
        or isinstance(task, AbsTaskAny2AnyRetrieval)
        or isinstance(task, AbsTaskInstructionRetrieval)
        or isinstance(task, MultiSubsetLoader)
        or isinstance(task, AbsTaskSpeedTask)
        or isinstance(task, AbsTaskAny2AnyMultiChoice)
        or isinstance(task, AbsTaskImageTextPairClassification)
    ):
        pytest.skip()
    with patch.object(task, "dataset_transform") as mock_dataset_transform:
        task.load_data()
        mock_load_dataset.assert_called()

        # They don't yet but should they so they can be expanded more easily?
        if not task.is_multilingual:
            mock_dataset_transform.assert_called_once()


@pytest.mark.test_datasets
@pytest.mark.flaky(
    reruns=3,
    reruns_delay=5,
    only_rerun=["AssertionError"],
    reason="May fail due to network issues",
)
@pytest.mark.parametrize("dataset_revision", dataset_revisions)
def test_dataset_on_hf(dataset_revision: tuple[str, str]):
    repo_id, revision = dataset_revision
    try:
        huggingface_hub.dataset_info(repo_id, revision=revision)
    except (
        huggingface_hub.errors.RepositoryNotFoundError,
        huggingface_hub.errors.RevisionNotFoundError,
    ):
        assert False, f"Dataset {repo_id} - {revision} not available"
    except Exception as e:
        assert False, f"Dataset {repo_id} - {revision} failed with {e}"


def test_superseded_dataset_exists():
    tasks = mteb.get_tasks(exclude_superseded=False)
    for task in tasks:
        if task.superseded_by:
            assert task.superseded_by in TASKS_REGISTRY, (
                f"{task} is superseded by {task.superseded_by} but {task.superseded_by} is not in the TASKS_REGISTRY"
            )


def test_is_aggregate_property_correct():
    tasks = mteb.get_tasks()

    for task in tasks:
        assert task.is_aggregate == isinstance(task, AbsTaskAggregate)



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskReranking import AbsTaskReranking


class AskUbuntuDupQuestions(AbsTaskReranking):
    metadata = TaskMetadata(
        name="AskUbuntuDupQuestions",
        description="AskUbuntu Question Dataset - Questions from AskUbuntu with manual annotations marking pairs of questions as similar or non-similar",
        reference="https://github.com/taolei87/askubuntu",
        dataset={
            "path": "mteb/askubuntudupquestions-reranking",
            "revision": "2000358ca161889fa9c082cb41daa8dcfb161a54",
        },
        type="Reranking",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="map",
        date=None,
        domains=["Programming", "Web"],
        task_subtypes=None,
        license=None,
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        prompt="Retrieve duplicate questions from AskUbuntu forum",
        bibtex_citation="""@article{wang-2021-TSDAE,
    title = "TSDAE: Using Transformer-based Sequential Denoising Auto-Encoderfor Unsupervised Sentence Embedding Learning",
    author = "Wang, Kexin and Reimers, Nils and  Gurevych, Iryna",
    journal= "arXiv preprint arXiv:2104.06979",
    month = "4",
    year = "2021",
    url = "https://arxiv.org/abs/2104.06979",
}""",
    )



from __future__ import annotations

from functools import partial
from typing import Any

import torch
from PIL import Image
from torch.utils.data import DataLoader
from tqdm import tqdm
from transformers import AutoImageProcessor, AutoModel

from mteb.encoder_interface import PromptType
from mteb.model_meta import ModelMeta


class DINOModelWrapper:
    """A wrapper class for DINO models that supports image encoding.
    Text encoding and text-image fusion are not supported.
    """

    def __init__(
        self,
        model_name: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        self.model_name = model_name
        self.device = device
        self.model = AutoModel.from_pretrained(model_name).to(self.device)
        self.processor = AutoImageProcessor.from_pretrained(model_name)

    @staticmethod
    def get_text_embeddings(
        texts: list[str],
        *,
        task_name: str | None = None,
        prompt_type: PromptType | None = None,
        batch_size: int = 32,
        **kwargs: Any,
    ):
        raise ValueError("DINO models only support image encoding.")

    def get_image_embeddings(
        self,
        images: list[Image.Image] | DataLoader,
        *,
        task_name: str | None = None,
        prompt_type: PromptType | None = None,
        batch_size: int = 32,
        pooling="cls",
        **kwargs: Any,
    ):
        all_image_embeddings = []

        if isinstance(images, DataLoader):
            with torch.no_grad():
                for batch in tqdm(images):
                    inputs = self.processor(images=batch, return_tensors="pt")
                    inputs = {k: v.to(self.device) for k, v in inputs.items()}
                    image_outputs = self.model(**inputs)
                    features = image_outputs.last_hidden_state
                    if pooling == "cls":
                        features = features[:, 0, :]  # TODO: confirm best practice
                    elif pooling == "mean":
                        features = features.mean(dim=1)
                    else:
                        raise ValueError(
                            "Pooling methods not implemented. Use cls or mean."
                        )
                    all_image_embeddings.append(features.cpu())
        else:
            with torch.no_grad():
                for i in tqdm(range(0, len(images), batch_size)):
                    batch_images = images[i : i + batch_size]
                    inputs = self.processor(images=batch_images, return_tensors="pt")
                    inputs = {k: v.to(self.device) for k, v in inputs.items()}
                    image_outputs = self.model(**inputs)
                    features = image_outputs.last_hidden_state
                    if pooling == "cls":
                        features = features[:, 0, :]
                    elif pooling == "mean":
                        features = features.mean(dim=1)
                    else:
                        raise ValueError(
                            "Pooling methods not implemented. Use cls or mean."
                        )
                    all_image_embeddings.append(features.cpu())

        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    @staticmethod
    def calculate_probs(text_embeddings, image_embeddings):
        raise ValueError("DINO models only support image encoding.")

    def get_fused_embeddings(
        self,
        texts: list[str] = None,
        images: list[Image.Image] | DataLoader = None,
        fusion_mode="sum",
        **kwargs: Any,
    ):
        if texts is None and images is None:
            raise ValueError("images must be provided for DINO models")

        text_embeddings = None
        image_embeddings = None

        if texts is not None:
            text_embeddings = self.get_text_embeddings(texts, **kwargs)

        if images is not None:
            image_embeddings = self.get_image_embeddings(images, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            raise ValueError("DINO models only support image encoding.")
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings


dinov2_training_datasets = {
    # LVD-142M
    #  ImageNet-22k
}


dinov2_small = ModelMeta(
    loader=partial(
        DINOModelWrapper,
        model_name="facebook/dinov2-small",
    ),
    name="facebook/dinov2-small",
    languages=["eng-Latn"],
    revision="ed25f3a31f01632728cabb09d1542f84ab7b0056",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=22_100_000,
    memory_usage_mb=84,
    max_tokens=None,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/dinov2-small",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
)

dinov2_base = ModelMeta(
    loader=partial(
        DINOModelWrapper,
        model_name="facebook/dinov2-base",
    ),
    name="facebook/dinov2-base",
    languages=["eng-Latn"],
    revision="f9e44c814b77203eaa57a6bdbbd535f21ede1415",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=86_600_000,
    memory_usage_mb=330,
    max_tokens=None,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/dinov2-base",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
)

dinov2_large = ModelMeta(
    loader=partial(
        DINOModelWrapper,
        model_name="facebook/dinov2-large",
    ),
    name="facebook/dinov2-large",
    languages=["eng-Latn"],
    revision="47b73eefe95e8d44ec3623f8890bd894b6ea2d6c",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=304_000_000,
    memory_usage_mb=1161,
    max_tokens=None,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/dinov2-large",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
)

dinov2_giant = ModelMeta(
    loader=partial(
        DINOModelWrapper,
        model_name="facebook/dinov2-giant",
    ),
    name="facebook/dinov2-giant",
    languages=["eng-Latn"],
    revision="611a9d42f2335e0f921f1e313ad3c1b7178d206d",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=1_140_000_000,
    memory_usage_mb=4335,
    max_tokens=None,
    embed_dim=1536,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/dinov2-giant",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
)



from __future__ import annotations

import numpy as np
import torch


def normalize_embeddings_to_numpy(
    embeddings: torch.Tensor | np.ndarray | list[np.ndarray] | list[torch.Tensor],
) -> np.ndarray:
    """Normalize embeddings to be numpy arrays


    Args:
        embeddings: embeddings to normalize

    Returns:
        Normalized embeddings
    """
    if isinstance(embeddings, torch.Tensor):
        embeddings = embeddings.cpu().detach().float().numpy()
    elif isinstance(embeddings, list):
        if isinstance(embeddings[0], torch.Tensor):
            embeddings = [
                embedding.cpu().detach().float().numpy() for embedding in embeddings
            ]
        elif isinstance(embeddings[0], np.ndarray):
            embeddings = embeddings

    numpy_embeddings = np.array(embeddings)

    return numpy_embeddings



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskImageMultilabelClassification import (
    AbsTaskImageMultilabelClassification,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


# NOTE: In the paper, this is grouped with linear probe tasks.
# See https://github.com/embeddings-benchmark/mteb/pull/2035#issuecomment-2661626309.
class VOC2007Classification(AbsTaskImageMultilabelClassification):
    metadata = TaskMetadata(
        name="VOC2007",
        description="Classifying bird images from 500 species.",
        reference="http://host.robots.ox.ac.uk/pascal/VOC/",
        dataset={
            "path": "HuggingFaceM4/pascal_voc",
            "name": "voc2007_main",
            "revision": "dbafdb9e1506c9c419c5c4672e409463cd21ba50",
            "trust_remote_code": True,
        },
        type="ImageClassification",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="lrap",
        date=(
            "2005-01-01",
            "2014-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation="""@Article{Everingham10,
            author = "Everingham, M. and Van~Gool, L. and Williams, C. K. I. and Winn, J. and Zisserman, A.",
            title = "The Pascal Visual Object Classes (VOC) Challenge",
            journal = "International Journal of Computer Vision",
            volume = "88",
            year = "2010",
            number = "2",
            month = jun,
            pages = "303--338",
        }""",
        descriptive_stats={
            "n_samples": {"test": 4952},
            "avg_character_length": {"test": 431.4},
        },
    )

    # Override default column name in the subclass
    label_column_name: str = "classes"

    # To be removed when we want full results
    n_experiments: int = 5



from __future__ import annotations

import logging

import datasets

from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

_EVAL_LANGS = {
    "bengali": ["ben-Beng"],
    "english": ["eng-Latn"],
    "finnish": ["fin-Latn"],
    "russian": ["rus-Cyrl"],
    "korean": ["kor-Kore"],
    "japanese": ["jpn-Jpan"],
    "telugu": ["tel-Telu"],
    "thai": ["tha-Thai"],
    "swahili": ["swa-Latn"],
    "arabic": ["ara-Arab"],
    "indonesian": ["ind-Latn"],
}
_EVAL_SPLIT = "test"

logger = logging.getLogger(__name__)


def _load_data_retrieval(
    path: str, langs: list, splits: str, cache_dir: str = None, revision: str = None
):
    corpus = {lang: {split: {} for split in splits} for lang in langs}
    queries = {lang: {split: {} for split in splits} for lang in langs}
    relevant_docs = {lang: {split: {} for split in splits} for lang in langs}

    split = _EVAL_SPLIT

    for lang in langs:
        qrels_data = datasets.load_dataset(
            path,
            name=f"{lang}-qrels",
            cache_dir=cache_dir,
            revision=revision,
            trust_remote_code=True,
        )[split]

        for row in qrels_data:
            query_id = row["query-id"]
            doc_id = row["corpus-id"]
            score = row["score"]
            if query_id not in relevant_docs[lang][split]:
                relevant_docs[lang][split][query_id] = {}
            relevant_docs[lang][split][query_id][doc_id] = score

        corpus_data = datasets.load_dataset(
            path,
            name=f"{lang}-corpus",
            cache_dir=cache_dir,
            revision=revision,
            trust_remote_code=True,
        )["train"]

        for row in corpus_data:
            doc_id = row["_id"]
            doc_title = row["title"]
            doc_text = row["text"]
            corpus[lang][split][doc_id] = {"title": doc_title, "text": doc_text}

        queries_data = datasets.load_dataset(
            path,
            name=f"{lang}-queries",
            cache_dir=cache_dir,
            revision=revision,
            trust_remote_code=True,
        )[split]

        for row in queries_data:
            query_id = row["_id"]
            query_text = row["text"]
            queries[lang][split][query_id] = query_text

        queries = queries
        logger.info("Loaded %d %s Queries.", len(queries), split.upper())

    return corpus, queries, relevant_docs


class MrTidyRetrieval(MultilingualTask, AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MrTidyRetrieval",
        description="Mr. TyDi is a multi-lingual benchmark dataset built on TyDi, covering eleven typologically diverse languages. It is designed for monolingual retrieval, specifically to evaluate ranking with learned dense representations.",
        reference="https://huggingface.co/datasets/castorini/mr-tydi",
        dataset={
            "path": "mteb/mrtidy",
            "revision": "fc24a3ce8f09746410daee3d5cd823ff7a0675b7",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2023-11-01", "2024-05-15"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="cc-by-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@article{mrtydi,
              title={{Mr. TyDi}: A Multi-lingual Benchmark for Dense Retrieval},
              author={Xinyu Zhang and Xueguang Ma and Peng Shi and Jimmy Lin},
              year={2021},
              journal={arXiv:2108.08787},
        }""",
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_data_retrieval(
            path=self.metadata_dict["dataset"]["path"],
            langs=self.hf_subsets,
            splits=self.metadata_dict["eval_splits"],
            cache_dir=kwargs.get("cache_dir", None),
            revision=self.metadata_dict["dataset"]["revision"],
        )

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class GLDv2I2TRetrieval(AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="GLDv2I2TRetrieval",
        description="Retrieve names of landmarks based on their image.",
        reference="https://openaccess.thecvf.com/content_CVPR_2020/html/Weyand_Google_Landmarks_Dataset_v2_-_A_Large-Scale_Benchmark_for_Instance-Level_CVPR_2020_paper.html",
        dataset={
            "path": "JamieSJS/gld-v2-i2t",
            "revision": "d8c3e53160860f76de73ed3041a8593672fe5928",
        },
        type="Any2AnyRetrieval",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2017-01-01", "2017-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""@InProceedings{Weyand_2020_CVPR,
author = {Weyand, Tobias and Araujo, Andre and Cao, Bingyi and Sim, Jack},
title = {Google Landmarks Dataset v2 - A Large-Scale Benchmark for Instance-Level Recognition and Retrieval},
booktitle = {Proceedings of the IEEE/CVF Conference on Computer Vision and Pattern Recognition (CVPR)},
month = {June},
year = {2020}
}""",
        descriptive_stats={
            "n_samples": {"test": 1972},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 0.0,
                    "num_documents": 674,
                    "num_queries": 1972,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

import logging
import platform
import time
import warnings
from pathlib import Path

import numpy as np

from mteb.encoder_interface import Encoder
from mteb.load_results.task_results import ScoresDict

from .AbsTask import AbsTask

logger = logging.getLogger(__name__)


WORDS_IN_UGLY_DUCKLING = 3633


class AbsTaskSpeedTask(AbsTask):
    """Abstract class for speed tasks (e.g. CPU, GPU)."""

    num_loops = 7
    device = "cpu"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def load_data(self, **kwargs):
        """Reads the text 'The Ugly Duckling' as the `test` split with a `text` column."""
        if self.data_loaded:
            return
        file_path = Path(__file__).parent / "the_ugly_duckling.txt"
        with file_path.open("r") as f:
            text = f.read()
        self.dataset = {"test": {"text": text.split("\n\n")}}
        self.data_loaded = True

    def _get_time_taken(self, model: Encoder, data_split) -> float:
        start = time.time()
        model.encode(
            data_split["text"], device=self.device, task_name=self.metadata.name
        )
        time_taken = time.time() - start
        return time_taken

    def get_system_info(self) -> dict[str, str]:
        """Returns a dictionary with system information."""
        try:
            import GPUtil
            import psutil
        except ImportError as e:
            raise ImportError(
                "GPUtil and psutil is not installed. Please install them `pip install GPUtil psutil` or `pip install mteb[speedtask]`"
            ) from e

        info = {}
        info["platform"] = platform.system()
        info["platform_release"] = platform.release()
        info["platform_version"] = platform.version()
        info["architecture"] = platform.machine()
        info["processor"] = platform.processor()
        info["ram"] = (
            str(round(psutil.virtual_memory().total / (1024.0**3))) + " GB"
        )  ## Convert from Bytes
        info["physical_cores"] = psutil.cpu_count(logical=False)
        info["total_cores"] = psutil.cpu_count(logical=True)

        ## NOTE: Currently works on nvidia GPUs only.
        if self.device != "cpu":
            import GPUtil

            gpus = GPUtil.getGPUs() or []
            info["num_gpus"] = len(gpus)
            list_gpus = []
            for gpu in gpus:
                list_gpus.append(
                    {
                        "gpu_name": gpu.name,
                        "gpu_total_memory": f"{gpu.memoryTotal / 1024.0} GB",
                    }
                )
            info["gpu_info"] = list_gpus
        return info

    def _evaluate_subset(self, model: Encoder, data_split, **kwargs) -> ScoresDict:
        warnings.warn(
            "SpeedTask is deprecated and will be removed in `v2`.",
            DeprecationWarning,
        )
        model.encode(
            ["encode this"], device=self.device, task_name=self.metadata.name
        )  # ensure model is loaded

        timings = []
        for _ in range(self.num_loops):
            time_taken = self._get_time_taken(model, data_split)
            timings.append(time_taken)

        time_mean = np.mean(timings)
        time_std = np.std(timings)
        scores = {
            "time_mean": time_mean,
            "time_std": time_std,
            "timings": timings,
            "avg_words_per_sec": WORDS_IN_UGLY_DUCKLING / time_mean,
            **self.get_system_info(),
        }
        self._add_main_score(scores)
        return scores

    def _add_main_score(self, scores) -> None:
        scores["main_score"] = scores[self.metadata.main_score]

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> dict[str, float]:
        pass



from __future__ import annotations

import hashlib
import json
import logging
import os
import random
import string
import time
from functools import partial
from typing import Any

import numpy as np
import requests

from mteb.model_meta import ModelMeta
from mteb.models.bge_models import bge_full_data
from mteb.models.e5_instruct import E5_MISTRAL_TRAINING_DATA
from mteb.models.wrapper import Wrapper

conan_zh_datasets = {
    "BQ": ["train"],
    "LCQMC": ["train"],
    "PAWSX": ["train"],
    "STS-B": ["train"],
    "DuRetrieval": ["train"],
    "AFQMC": ["train"],
    "Cmnli": ["train"],
    "Ocnli": ["train"],
    "T2Retrieval": ["train"],
    "T2Reranking": ["train"],
    "MMarcoReranking": ["train"],
    "CMedQAv2-reranking": ["train"],
}

logger = logging.getLogger(__name__)


class RateLimiter:
    def __init__(self, qps, max_retries=3):
        self.qps = qps
        self.min_interval = 1.0 / qps
        self.last_request_time = 0
        self.max_retries = max_retries

    def wait(self):
        """Simple rate limiting logic"""
        current_time = time.time()
        time_since_last = current_time - self.last_request_time
        if time_since_last < self.min_interval:
            sleep_time = self.min_interval - time_since_last
            time.sleep(sleep_time)
        self.last_request_time = time.time()

    def execute_with_retry(self, func, *args, **kwargs):
        """Execute a function with retry logic

        Args:
            func: The function to execute
            *args: Arguments to pass to the function
            **kwargs: Arguments to pass to the function

        Returns:
            The result of the function execution

        Raises:
            Exception: When max retries are reached and still failing
        """
        retries = 0
        while retries < self.max_retries:
            self.wait()
            try:
                return func(*args, **kwargs)
            except Exception as e:
                retries += 1
                if retries < self.max_retries:
                    sleep_time = 10 * retries
                    logger.warning(
                        f"Request failed (attempt {retries}/{self.max_retries}), "
                        f"sleeping for {sleep_time}s. Error: {str(e)}"
                    )
                    time.sleep(sleep_time)
                else:
                    logger.error(f"Max retries reached. Last error: {str(e)}")
                    raise


class Client:
    def __init__(self, ak, sk, url, timeout=30):
        self.ak = ak
        self.sk = sk
        self.url = url
        self.timeout = timeout
        self.rate_limiter = RateLimiter(qps=5, max_retries=3)

    def _random_password(self, size=40, chars=None):
        if chars is None:
            chars = string.ascii_uppercase + string.ascii_lowercase + string.digits
        random_chars = random.SystemRandom().choice
        return "".join(random_chars(chars) for _ in range(size))

    def __signature(self, random_str, time_stamp):
        params_str = f"{self.ak}:{time_stamp}:{random_str}:{self.sk}"
        encoded_params_str = params_str.encode("utf-8")
        return hashlib.md5(encoded_params_str).hexdigest()

    def get_signature(self):
        timestamp = int(time.time())
        random_str = self._random_password(20)
        sig = self.__signature(random_str, timestamp)
        params = {
            "timestamp": timestamp,
            "random": random_str,
            "app_id": self.ak,
            "sign": sig,
        }
        return params

    def _do_request(self, text):
        """Execute the actual request without retry logic"""
        params = self.get_signature()
        params["body"] = text
        params["content_id"] = f"test_{int(time.time())}"
        headers = {"Content-Type": "application/json"}

        rsp = requests.post(
            self.url, data=json.dumps(params), timeout=self.timeout, headers=headers
        )
        result = rsp.json()

        if rsp.status_code != 200:
            raise Exception(
                f"API request failed with status {rsp.status_code}: {result}"
            )

        return result

    def embed(self, text):
        """Embed text using the server with rate limiting and retry logic

        Args:
            text: The input text to embed

        Returns:
            dict: Response containing embedding
        """
        # Use rate_limiter to execute the request, handling rate limiting and retries
        return self.rate_limiter.execute_with_retry(self._do_request, text)


class ConanWrapper(Wrapper):
    def __init__(
        self,
        model_name: str,
        **kwargs,
    ) -> None:
        AK = os.getenv("CONAN_AK")
        SK = os.getenv("CONAN_SK")
        if not AK or not SK:
            raise ValueError("CONAN_AK and CONAN_SK environment variables must be set")

        self.client = Client(ak=AK, sk=SK, url="https://ai.om.qq.com/api/conan/v2")
        self.model_name = model_name

    def encode(
        self,
        sentences: list[str],
        **kwargs: Any,
    ) -> np.ndarray:
        embeddings = []

        for sentence in sentences:
            try:
                result = self.client.embed(sentence)
                if "embedding" not in result:
                    raise ValueError(f"No embedding in response: {result}")
                embeddings.append(result["embedding"])
            except Exception as e:
                logger.error(f"Failed to embed sentence: {str(e)}")
                raise

        return np.array(embeddings)


Conan_embedding_v2 = ModelMeta(
    name="TencentBAC/Conan-embedding-v2",
    revision="e5c87c63889630bca87486f6a2645ed97c5ddb17",
    release_date="2025-04-10",
    languages=[
        "eng-Latn",
        "zho-Hans",
    ],
    loader=partial(  # type: ignore
        ConanWrapper,
        model_name="Conan-embedding-v2",
    ),
    max_tokens=32768,
    embed_dim=3584,
    open_weights=False,
    n_parameters=None,
    memory_usage_mb=None,
    license="apache-2.0",
    reference="https://huggingface.co/TencentBAC/Conan-embedding-v2",
    similarity_fn_name="cosine",
    framework=["API"],
    use_instructions=True,
    training_datasets={
        **E5_MISTRAL_TRAINING_DATA,
        **bge_full_data,
        **conan_zh_datasets,
    },
    public_training_code=None,
    public_training_data=None,
)



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskSTS import AbsTaskSTS

N_SAMPLES = 1000


class FinParaSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="FinParaSTS",
        dataset={
            "path": "TurkuNLP/turku_paraphrase_corpus",
            "revision": "e4428e399de70a21b8857464e76f0fe859cabe05",
            "name": "plain",
            "trust_remote_code": True,
        },
        description="Finnish paraphrase-based semantic similarity corpus",
        reference="https://huggingface.co/datasets/TurkuNLP/turku_paraphrase_corpus",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["fin-Latn"],
        main_score="cosine_spearman",
        date=("2017-01-01", "2021-12-31"),
        domains=["News", "Subtitles", "Written"],
        task_subtypes=None,
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
        @inproceedings{kanerva-etal-2021-finnish,
            title = "{F}innish Paraphrase Corpus",
            author = {Kanerva, Jenna  and
            Ginter, Filip  and
            Chang, Li-Hsin  and
            Rastas, Iiro  and
            Skantsi, Valtteri  and
            Kilpel{\"a}inen, Jemina  and
            Kupari, Hanna-Mari  and
            Saarni, Jenna  and
            Sev{\'o}n, Maija  and
            Tarkka, Otto},
            editor = "Dobnik, Simon  and
            {\\O}vrelid, Lilja",
            booktitle = "Proceedings of the 23rd Nordic Conference on Computational Linguistics (NoDaLiDa)",
            month = may # " 31--2 " # jun,
            year = "2021",
            address = "Reykjavik, Iceland (Online)",
            publisher = {Link{\"o}ping University Electronic Press, Sweden},
            url = "https://aclanthology.org/2021.nodalida-main.29",
            pages = "288--298",
        }
        """,
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 2
        metadata_dict["max_score"] = 4
        return metadata_dict

    def dataset_transform(self):
        self.dataset = self.dataset.shuffle(seed=self.seed)
        for split in self.dataset:
            self.dataset[split] = self.dataset[split].select(range(N_SAMPLES))
        rename_dict = {"text1": "sentence1", "text2": "sentence2", "label": "score"}
        self.dataset = self.dataset.rename_columns(rename_dict)
        self.dataset = self.dataset.select_columns(list(rename_dict.values()))
        self.dataset = self.dataset.map(lambda x: {"score": int(x["score"][0])})



from __future__ import annotations

import itertools

from datasets import Dataset, DatasetDict

from mteb.abstasks.AbsTaskClustering import AbsTaskClustering
from mteb.abstasks.AbsTaskClusteringFast import (
    AbsTaskClusteringFast,
    check_label_distribution,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class MedrxivClusteringP2PFast(AbsTaskClusteringFast):
    metadata = TaskMetadata(
        name="MedrxivClusteringP2P.v2",
        description="Clustering of titles+abstract from medrxiv across 51 categories.",
        reference="https://api.medrxiv.org/",
        dataset={
            "path": "mteb/medrxiv-clustering-p2p",
            "revision": "e7a26af6f3ae46b30dde8737f02c07b1505bcc73",
        },
        type="Clustering",
        category="p2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2021-01-01", "2022-05-10"),
        domains=["Academic", "Medical", "Written"],
        task_subtypes=["Thematic clustering"],
        license="https://www.medrxiv.org/content/about-medrxiv",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",
        prompt="Identify the main category of Medrxiv papers based on the titles and abstracts",
        adapted_from=["MedrxivClusteringP2P"],
    )

    def dataset_transform(self):
        ds = {}
        for split in self.metadata.eval_splits:
            labels = list(itertools.chain.from_iterable(self.dataset[split]["labels"]))
            sentences = list(
                itertools.chain.from_iterable(self.dataset[split]["sentences"])
            )
            check_label_distribution(self.dataset[split])
            ds[split] = Dataset.from_dict({"labels": labels, "sentences": sentences})
        self.dataset = DatasetDict(ds)


class MedrxivClusteringP2P(AbsTaskClustering):
    superseded_by = "MedrxivClusteringP2P.v2"
    metadata = TaskMetadata(
        name="MedrxivClusteringP2P",
        description="Clustering of titles+abstract from medrxiv. Clustering of 10 sets, based on the main category.",
        reference="https://api.medrxiv.org/",
        dataset={
            "path": "mteb/medrxiv-clustering-p2p",
            "revision": "e7a26af6f3ae46b30dde8737f02c07b1505bcc73",
        },
        type="Clustering",
        category="p2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2021-01-01", "2022-05-10"),
        domains=["Academic", "Written"],
        task_subtypes=["Thematic clustering"],
        license="https://www.medrxiv.org/content/about-medrxiv",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",
        prompt="Identify the main category of Medrxiv papers based on the titles and abstracts",
    )
